//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import PT from '../PT';
import _ from 'lodash';
import Util from '../utils';

//---------------------------------------------------------------------------//

const discoveryApi = new PT.Netcon_Sdk.DiscoveryApi();

const getNetworkDeviceByDiscoveryId = (dispatch, id) => {
    discoveryApi.getNetworkDeviceByDiscoveryId(id, (err, data, res) => {
        Util.responseHandler(err, data, res, dispatch, ()=>{
            let result = JSON.parse(res.text);
            if(result){
                dispatch({
                    type: 'DISCOVERY_DISCOVERED_DEVICES',
                    payload: {
                        list: result.response
                    }
                });
            }
        });
    });
};

const getDiscovery = (dispatch) => {
    discoveryApi.getDiscovery((err, data, res) => {
        Util.responseHandler(err, data, res, dispatch, ()=>{
            let result = JSON.parse(res.text);
            if(result){
                if(_.isEmpty(result)){
                    result=[];
                }else{
                    result = result.response;
                }
                dispatch({
                    type: 'DISCOVERY_FETCH_ALL',
                    payload: {
                        list: result
                    }
                });
            }
        });
    });
};

const getDiscoveryById = (dispatch, id) => {
    discoveryApi.getDiscoveryById(id, (err, data, res) => {
        Util.responseHandler(err, data, res, dispatch, ()=>{
            let result = JSON.parse(res.text).response;
            if(result){
                dispatch({
                    type: 'DISCOVERY_FETCH',
                    payload: {
                        settings: result
                    }
                });
            }
        });
    });
};

const insertDiscovery = (dispatch, discovery) => {
    discoveryApi.insertDiscovery(discovery, (err, data, res) => {
        Util.responseHandler(err, data, res, dispatch, ()=>{
            getDiscovery(dispatch);
        });
    });
};

const updateDiscovery = (dispatch, discovery) => {
    discoveryApi.updateDiscovery(discovery, (err, data, res) => {
        Util.responseHandler(err, data, res, dispatch, ()=>{
            getDiscovery(dispatch);
        });
    });
};

const deleteDiscoveryById = (dispatch, id) => {
    discoveryApi.deleteDiscoveryById(id, (err, data, res) => {
        Util.responseHandler(err, data, res, dispatch, ()=>{
            getDiscovery(dispatch);
        });
    });
};

const start = (dispatch, discovery) => {
    discoveryApi.updateDiscovery({
        id: discovery.id,
        discoveryStatus: 'active'
    }, (err, data, res) => {

        Util.responseHandler(err, data, res, dispatch, ()=>{
            getDiscovery(dispatch);
        });
    });
};

const stop = (dispatch, discovery) => {
    discoveryApi.updateDiscovery({
        id: discovery.id,
        discoveryStatus: 'inactive'
    }, (err, data, res) => {
        Util.responseHandler(err, data, res, dispatch, ()=>{
            getDiscovery(dispatch);
        });
    });
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default { 
    getNetworkDeviceByDiscoveryId, 
    getDiscovery, 
    getDiscoveryById, 
    insertDiscovery, 
    updateDiscovery, 
    deleteDiscoveryById,
    start, 
    stop,
}

//---------------------------------------------------------------------------//