//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import PT from '../PT';
import Toast from '../components/Toast';
import Util from '../utils';
import _ from 'lodash';

//---------------------------------------------------------------------------//

const inventoryApi = new PT.Netcon_Sdk.InventoryApi();

const getNetworkDevices = (dispatch) => {
    inventoryApi.getNetworkDevices((err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            let result = JSON.parse(res.text);
            let list = [];
            if(_.isUndefined(result.response)){
                list = [];
            }else{
                list = result.response;
            }
            dispatch({
                type: 'INVENTORY_FETCH',
                payload: {
                    list: list
                }
            });
        });
    });
};

const deleteNetworkDeviceByIp = (dispatch, ip) => {

    if(ip===undefined){
        Toast.error('Cannot delete device');
        return;
    }

    inventoryApi.deleteNetworkDeviceByIp(ip, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            getNetworkDevices(dispatch);
        });
    });
};

const addNetworkDevice = (dispatch, device) => {
    inventoryApi.addNetworkDevice(device, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            getNetworkDevices(dispatch);
        });
    });
};

const updateNetworkDevice = (dispatch, device) => {
    inventoryApi.updateNetworkDevice(device, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            let result = JSON.parse(res.text);
            if(result){
                dispatch({
                    type: 'INVENTORY_EDIT_DEVICE',
                    payload: {
                        device: device
                    }
                });
                Toast.success('Updated Successfully');
            }
        });
    });
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default { 
    getNetworkDevices,
    deleteNetworkDeviceByIp,
    addNetworkDevice,
    updateNetworkDevice
};

//---------------------------------------------------------------------------//