//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import PT from '../PT';
import Util from '../utils';

//---------------------------------------------------------------------------//

const ticketApi = new PT.Netcon_Sdk.TicketApi();
const userApi = new PT.Netcon_Sdk.UserApi();

const addTicket = (dispatch, user, history) => {
    let payload = {
        'user': {...user }
    }
    ticketApi.addTicket(payload, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            let response = JSON.parse(res.text).response;
            let apiKey = response.serviceTicket;
    
            PT.setUser(user.username, apiKey);
            PT.setAuthTokenHeader(apiKey);

            dispatch({
                type: 'APP_SET_STATE',
                payload: {
                    apiKey: apiKey,
                    username: user.username
                }
            })
            history.push('/dashboard');
        });
    });
};

const addSetup = (dispatch, user, history) => {
    let payload = {
        'user': {...user }
    }
    userApi.addSetup(payload, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            dispatch({
                type: 'APP_SET_SETUPCOMPLETED',
                payload: {
                    setupCompleted: true
                }
            })
            history.push('/login');
        });
    });
};

const deleteTicket = (dispatch) => {

    let apiKey = PT.getApiKey();

    ticketApi.deleteTicket(apiKey, (err, item, res) => {
        
        PT.clearUser();

        Util.responseHandler(err, item, res, dispatch, ()=>{
            
            let response = JSON.parse(res.text).response;
        
            if(response.errorCode){
                dispatch({
                    type: 'MESSAGE_ERROR_WITH_CODE',
                    payload: {
                        code: response.errorCode,
                        message: response.message,
                        detail: response.detail
                    }
                });
            }

            else {
                PT.clearAuthTokenHeader()
                dispatch({
                    type: 'APP_LOGOUT'
                })
            }

        });

    });
};


//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default { 
    addTicket,
    deleteTicket, 
    addSetup 
};

//---------------------------------------------------------------------------//