//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import PT from '../PT';
import _ from 'lodash';
import Util from '../utils';

//---------------------------------------------------------------------------//

const pathtraceApi = new PT.Netcon_Sdk.FlowAnalysisApi();

const flowAnalysis = (dispatch) => {
    pathtraceApi.flowAnalysis((err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            let result = JSON.parse(res.text).response;
            if(result){
                if(_.isEmpty(result)){
                    result=[];
                }else{
                    result = result.flowAnalysisList;
                }
                dispatch({
                    type: 'PATHTRACE_FETCH_ALL',
                    payload: {
                        list: result
                    }
                });
            }
        });
    });
};

const getFullFlowAnalysisResult = (dispatch, id) => {
    pathtraceApi.getFullFlowAnalysisResult(id, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            let result = JSON.parse(res.text).response;
            if(result){
                dispatch({
                    type: 'PATHTRACE_DETAIL',
                    payload: {
                        detail: result
                    }
                });
            }
        });
    });
};

const initiateFlowAnalysis = (dispatch, config) => {

    let payload = {
        flowAnalysisRequest: {...config }
    }

    pathtraceApi.initiateFlowAnalysis(payload, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            let result = JSON.parse(res.text);
            if(result){
                flowAnalysis(dispatch);
            }
        });
    });
};

const deleteFlowAnalysis = (dispatch, id) => {
    pathtraceApi.deleteFlowAnalysis(id, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            let result = JSON.parse(res.text);
            if(result){
                dispatch({
                    type: 'PATHTRACE_DETAIL',
                    payload: {
                        detail: {}
                    }
                });
                flowAnalysis(dispatch);
            }
        });
    });
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default { 
    flowAnalysis, 
    getFullFlowAnalysisResult, 
    initiateFlowAnalysis, 
    deleteFlowAnalysis 
};

//---------------------------------------------------------------------------//