//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import PT from '../PT';
import Util from '../utils';
import _ from 'lodash';

//---------------------------------------------------------------------------//

const policyApi = new PT.Netcon_Sdk.QosApi();
const inventoryApi = new PT.Netcon_Sdk.InventoryApi();

const getFilterApplication = (dispatch, name) => {
    policyApi.getFilterApplication({name:name}, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            let response = JSON.parse(res.text);
            if(response){
                if(_.isEmpty(response)){
                    response=[];
                }else{
                    response = response.response;
                }
                dispatch({
                    type: 'QOS_SET_STATE',
                    payload: {
                        protocols: response
                    }
                });
            }
        });
    });
};

const getFilterPolicies = (dispatch, query) => {
    policyApi.getFilterPolicies(query, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            let response = JSON.parse(res.text);
            if(response){
                if(_.isEmpty(response)){
                    response=[];
                }else{
                    response = response.response;
                }
                dispatch({
                    type: 'QOS_SET_STATE',
                    payload: {
                        policies: response
                    }
                });
            }
        });
    });
};

const getPolicyTags = (dispatch) => {
    policyApi.getPolicyTags((err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            let response = JSON.parse(res.text);
            if(response){
                if(_.isEmpty(response)){
                    response=[];
                }else{
                    response = response.response;
                }
                dispatch({
                    type: 'QOS_SET_STATE',
                    payload: {
                        scopes: response
                    }
                });
            }
        });
    });
};

const addPolicy = (dispatch, policy) => {
    policyApi.addPolicy(policy, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            getFilterPolicies(dispatch, '');
        });
    });
};

const deletePolicy = (dispatch, id) => {
    policyApi.deletePolicy(id, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            getFilterPolicies(dispatch, '');
        });
    });
};

const addPolicyTag = (dispatch, config) => {
    let payload = {
        policyTagDTO: {...config}
    };
    policyApi.addPolicyTag(payload, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            getPolicyTagAssociation(dispatch);
        });
    });
};

const updatePolicy = (dispatch, config) => {
    policyApi.updatePolicy(config.id, config.payload, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            getFilterPolicies(dispatch, '');
        });
    });
};

const deletePolicyTag = (dispatch, policyTag) => {
    policyApi.deletePolicyTag(policyTag, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            getPolicyTagAssociation(dispatch);
        });
    });
};

const deletePolicyTagAssociationAndPolicy = (dispatch, policyTag) => {
    policyApi.deletePolicyTagAssociation(policyTag, '', (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            policyApi.deletePolicyTag(policyTag, (err, item, res) => {
                Util.responseHandler(err, item, res, dispatch, ()=>{
                    getPolicyTagAssociation(dispatch);
                });
            });
        });
    });
};

const getNetworkDevices = (dispatch) => {
    inventoryApi.getNetworkDevices((err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            let list = JSON.parse(res.text).response;
            if(list){
                dispatch({
                    type: 'INVENTORY_FETCH',
                    payload: {
                        list: list
                    }
                });
            }
        });
    });
};

const addPolicyTagAssociation = (dispatch, config) => {
    let payload = {
        policyTagAssociationDTO: {...config}
    };
    policyApi.addPolicyTagAssociation(payload, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            getPolicyTagAssociation(dispatch);
        });
    });
};

const updatePolicyTagAssociation = (dispatch, config) => {
    policyApi.deletePolicyTagAssociation(config.policyTag, '', (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            addPolicyTagAssociation(dispatch, config);
        });
    });
};

const deletePolicyTagAssociation = (dispatch, policyTag, deviceId) => {
    policyApi.deletePolicyTagAssociation(policyTag, deviceId, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            getPolicyTagAssociation(dispatch);
        });
    });
};

const getPolicyTagAssociation = (dispatch, query) => {
    policyApi.getPolicyTagAssociation(query, (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            let response = JSON.parse(res.text);
            if(response){
                if(_.isEmpty(response)){
                    response=[];
                }else{
                    response = response.response;
                }
                dispatch({
                    type: 'QOS_SET_STATE',
                    payload: {
                        scopes: response
                    }
                });
            }
        });
    });
};

const getPolicyTagsCount = (dispatch) => {
    policyApi.getPolicyTagsCount((err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            let response = JSON.parse(res.text);
            dispatch({
                type: 'QOS_SET_STATE',
                payload: {
                    scopeCount: response.response
                }
            });
        });
    });
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default { 
    getFilterApplication,
    getFilterPolicies,
    addPolicy,
    deletePolicy,
    updatePolicy,
    addPolicyTag,
    getPolicyTags,
    getNetworkDevices,
    deletePolicyTag,
    addPolicyTagAssociation,
    getPolicyTagAssociation,
    updatePolicyTagAssociation,
    deletePolicyTagAssociationAndPolicy,
    deletePolicyTagAssociation,
    getPolicyTagsCount
}

//---------------------------------------------------------------------------//