//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React, { useEffect } from "react";
import { HashRouter, Switch, Route } from "react-router-dom";
import { ThemeProvider } from "@material-ui/core/styles";
import { connect } from 'react-redux';

import NavBar from './views/Navbar/index';
import Login from './views/Login/index';
import Topology from './views/Topology/index';
import Inventory from './views/Inventory/index';
import Discovery from './views/Discovery/index';
import Host from './views/Host/index';
import Dashboard from './views/Dashboard/index';
import Policy from './views/Policy/index';
import Setting from './views/Setting/index';
import User from './views/User/index';
import Pathtrace from './views/Pathtrace/index';
import Assurance from './views/Assurance/index';
import Swagger from './views/Swagger/index';
import Credential from './views/Credential/index';
import PT from './PT';
import Message from './components/Message';
import Util from './utils'
import Banner from './components/Banner';

import muiTheme from './muiTheme';

//---------------------------------------------------------------------------//

const swagger_api = new PT.Netcon_Sdk.UserApi();

const fetchUser = (dispatch) => {
    swagger_api.getUser(PT.getUsername(), (err, item, res) => {
        Util.responseHandler(err, item, res, dispatch);
    });
};

const fetchSetupFlag = (dispatch) => {
    swagger_api.getSetup((err, item, res) => {
        Util.responseHandler(err, item, res, dispatch, ()=>{
            let response = JSON.parse(res.text).response;
            dispatch({
                type: 'APP_SET_SETUPCOMPLETED',
                payload: {
                    setupCompleted: response
                }
            });

            // Do a fetch to check if stored apikey is valid.

            if(PT.getApiKey().length>1){
                fetchUser(dispatch);
            }
        });

    });
};

//---------------------------------------------------------------------------//
// App Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    useEffect(() => {
        props.dispatchUtil.setupFlag();
        props.dispatchUtil.setAppState({
            username: PT.getUsername(),
            apiKey: PT.getApiKey()
        });
    },[props.dispatchUtil]); 
    
    const GetRoute = (RouteComponent) => {
        
        let apiKey = props.state.apiKey;

        if(!apiKey || apiKey === ''){
            return(<Login />);
        }else{
            return (
                <>
                    <NavBar />
                    <RouteComponent />
                </>
            );
        }
    };

    return (
        <>
            <Banner />
            <Message></Message>
            <ThemeProvider theme={muiTheme}>
                <HashRouter>
                    <Switch>
                        <Route path="/login">
                            <Login />
                        </Route>
                        <Route path="/dashboard">
                            {GetRoute(Dashboard)}
                        </Route>
                        <Route path="/inventory">
                            {GetRoute(Inventory)}
                        </Route>
                        <Route path="/discovery">
                            {GetRoute(Discovery)}
                        </Route>
                        <Route path="/host">
                            {GetRoute(Host)}
                        </Route>
                        <Route path="/topology">
                            {GetRoute(Topology)}
                        </Route>
                        <Route path="/pathtrace">
                            {GetRoute(Pathtrace)}
                        </Route>
                        <Route path="/setting">
                            {GetRoute(Setting)}
                        </Route>
                        <Route path="/user">
                            {GetRoute(User)}
                        </Route>
                        <Route path="/assurance">
                            {GetRoute(Assurance)}
                        </Route>
                        <Route path="/qos">
                            {GetRoute(Policy)}
                        </Route>
                        <Route path="/swagger">
                            {GetRoute(Swagger)}
                        </Route>
                        <Route path="/credential">
                            {GetRoute(Credential)}
                        </Route>  
                        <Route path="/">
                            {GetRoute(Dashboard)}
                        </Route>
                      
                    </Switch>
                </HashRouter>
            </ThemeProvider>
        </>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

const mapStateToProps = (state) => {
    return {
        state: state.app
    }
};

const mapDispatchToProps = (dispatch) => {
    return {
        dispatchUtil: {
            setupFlag: () => {
                fetchSetupFlag(dispatch);
            },
            setApiKey: (apiKey) => {
                dispatch({
                    type: 'APP_SET_APIKEY',
                    payload: {
                        apiKey: apiKey
                    }
                })
            },
            setAppState: (payload) => {
                dispatch({
                    type: 'APP_SET_STATE',
                    payload: {
                        apiKey: payload.apiKey,
                        username: payload.username
                    }
                })
            }
        }
    }
};

export default connect(
    mapStateToProps,
    mapDispatchToProps
)(Component);

//---------------------------------------------------------------------------//