//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import { faExclamationTriangle, faTools } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import { connect } from 'react-redux';

//---------------------------------------------------------------------------//

library.add(faExclamationTriangle, faTools);

const bannerStyles = {
    info: {background: '#d1ecf1' ,color: '#0c5460', icon: 'exclamation-triangle'},
    warning: {background: '#fff3cd' ,color: '#c49345', icon: 'exclamation-triangle'},
    construction: {background: '#fff3cd' ,color: '#c49345', icon: 'tools'}
}

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const getBanner = (style, message) => {

        style = bannerStyles[props.state.type]

        if(props.state.show){
            return (
                <div
                    style={{
                        position:'relative',
                        textAlign:'center',
                        whiteSpace:'nowrap',
                        color: 'white',
                        background: style.background,
                        fontSize:16,
                        paddingTop: 10,
                        height: 30
                    }}
                >
                    <FontAwesomeIcon 
                        icon={style.icon}
                        size='lg' 
                        fixedWidth 
                        style={{
                            color: style.color
                        }}/>
                    <span
                        style={{
                            marginLeft: 5,
                            color: style.color
                        }}
                    >
                        {props.state.message}
                    </span>
                </div>
            );
        }
        else{
            return null;
        }
    }

    return (
        <>
            {getBanner(bannerStyles['construction'], 'Dashboard Work In Progress')}
        </>
    );
}


//---------------------------------------------------------------------------//
// Redux Actions
//---------------------------------------------------------------------------//

const mapStateToProps = state => ({
    state: state.banner
});

const mapDispatchToProps = dispatch => ({
    dispatch: {
        close: () => {
            dispatch({
                type: 'BANNER_CLOSE',
                payload: {
                    open: false
                }
            })
        },
    }
});

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default connect(
    mapStateToProps,
    mapDispatchToProps
)(Component);

//---------------------------------------------------------------------------//