//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Button from '@material-ui/core/Button';
import { faExclamationTriangle } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogTitle from '@material-ui/core/DialogTitle';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import { connect } from 'react-redux';

//---------------------------------------------------------------------------//

library.add(faExclamationTriangle);

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    let message = props.state.message;

    const handleClose = () => {
        props.dispatch.close();
    };

    return (
        <Dialog 
            open={props.state.open} 
            onClose={handleClose} 
            aria-labelledby='message-dialog-title' 
            maxWidth='md' 
            scroll='paper' 
            fullWidth
        >
            <DialogTitle id='message-dialog-title'>
                <div
                    style={{
                        color:'red'
                    }}
                >
                    <FontAwesomeIcon icon='exclamation-triangle' size='lg'
                        style={{
                            marginRight: 8
                        }}
                    />
                    <span>
                        Error
                    </span>
                </div>
            </DialogTitle>
            <DialogContent>
                <div style={{
                    display:'block',
                    textAlign:'center'
                }}>
                    <span>{message} </span>
                </div>
            </DialogContent>
            <DialogActions>
                <Button
                    id='message-dialog-confirm'
                    onClick={handleClose} 
                    color='primary'
                >
                    Okay
                </Button>
            </DialogActions>
        </Dialog>
    );
};

//---------------------------------------------------------------------------//
// Redux Actions
//---------------------------------------------------------------------------//

const mapStateToProps = state => ({
    state: state.message
});

const mapDispatchToProps = dispatch => ({
    dispatch: {
        close: () => {
            dispatch({
                type: 'MESSAGE_CLOSE',
                payload: {
                    open: false
                }
            })
        },
    }
});

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default connect(
    mapStateToProps,
    mapDispatchToProps
)(Component);

//---------------------------------------------------------------------------//