//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Grid from '@material-ui/core/Grid';
import {Pie} from 'react-chartjs-2';
import 'chartjs-plugin-labels';

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const options = {
    maintainAspectRatio: false,
    responsive: false,
    legend: {
        position: 'left',
        labels: {
            boxWidth: 10
        }
    },
    plugins:{
        labels: {
            precision:2,
            fontColor: '#fff',
            fontSize: 12,
            render: (args) => {
                return `${Math.floor(args.percentage)}%`;
            }
        }
    }
};

const Component = (props) => {

    let ratio = props.healthyRatio.split(':')
    let count = ratio[0];
    let total = ratio[1];

    const data = {
        labels: props.labels,
        datasets: [{
            data: [count, total-count],
            backgroundColor: [
                '#17b341',
                '#f01650',
            ],
            hoverBackgroundColor: [
                '#17b341',
                '#f01650',
            ]
        }]
    };

    return (
        <>
            <Grid 
                container
                direction='row'
                justify='center'
                alignItems='flex-start'
            >
                <Grid item xs={6}>
                    <div
                        style={{
                            fontWeight:'bold',
                            fontSize:18,
                            textAlign:'left',
                            marginTop:20
                        }}
                    >
                        <span id={`assurance-ratio-deviceType-${props.deviceType}`}>{props.deviceType}</span>
                    </div>
                    <div
                        style={{
                            textAlign:'left',
                            color:'#495057',
                            display:'block',
                            marginTop:3
                        }}
                    >
                        <span>{'Healthy Percentage: '}</span>
                        <b id={`assurance-ratio-healthyPercentage-${props.deviceType}`}>{props.healthyPercentage}%</b>
                    </div>
                    <div
                        style={{
                            textAlign:'left',
                            color:'#495057',
                            display:'block',
                            marginTop:3
                        }}
                    >
                        <span>{'Healthy Ratio: '}</span>
                        <b id={`assurance-ratio-healthyRatio-${props.deviceType}`}>{props.healthyRatio}</b>
                    </div>
                </Grid>
                <Grid item xs={6}>
                    <div
                        id={`assurance-ratio-pie-${props.deviceType}`}
                    >
                        <Pie data={data} height={150} width={200} options={options}/>
                    </div>
                </Grid>
            </Grid>
            <hr 
                style={{
                    display:props.isLast?'none':'show'
                }}
            />
        </>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//