//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Grid from '@material-ui/core/Grid';
import { Line } from 'react-chartjs-2';

//---------------------------------------------------------------------------//

const getData = (props) => {
    return {
        labels: [],
        datasets: [{
            data: props.data,
            label: 'Point',
            type: 'line',
            pointRadius: 0,
            fill: true,
            borderColor : '#36eb7f',
            backgroundColor: '#b8f8d2',
            borderWidth: 2
        }]
    };
};

const getOptions = (props) => {
    
    return {
        maintainAspectRatio: false,
        responsive: true,
        legend: {
            display: false,
        },
        scales: {
            xAxes: [{
                type: 'time',
                ticks: {
                    display:true,
                    maxTicksLimit: 5
                },
                gridLines: {
                    display:false
                },
                time: {
                    displayFormats: {
                        second: 'H:mm:ss'
                    }
                }
            }],
            yAxes: [{
                gridLines: {
                    display:true
                },
                ticks: {
                    fontColor: 'black',
                    stepSize: 25,
                    beginAtZero: true,
                    min: 0,
                    max: 100,
                    callback: (value) => {
                        return value + '%'; 
                    }
                }
            }]
        },
        tooltips: {
            intersect: false,
            mode: 'index',
            callbacks: {
                label: function(tooltipItem, myData) {
                    return `Health Level: ${parseFloat(tooltipItem.value).toFixed(2)}%`;
                }
            }
        },
        // Chart optimizations
        elements: {
            line: {
                tension: 0, // disables bezier curves
                fill: false,
                stepped: false,
                borderDash: []
            },
            point: {
                radius: 0 // default to disabled in all datasets
            }
        },
        spanGaps: true
    };


};

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    return (
        <Grid 
            container
            direction='row'
            justify='flex-start'
            alignItems='flex-start'
        >
            <Grid item xs={6}>
                <div
                    style={{
                        color:'#495057',
                        display: 'block',
                        textAlign:'center',
                        marginBottom:10
                    }}
                >
                    <span 
                        style={{
                            fontSize: 14,
                            fontWeight: 'bold',
                        }}>
                            {props.name}
                    </span>
                    <span 
                        style={{
                            fontSize: 24,
                            display: 'block',
                            fontWeight: 'bold',
                        }}
                        id={`assurance-health-total-${props.name}`}
                    >
                        {props.total}
                    </span>
                </div>
            </Grid>
            <Grid item xs={6}>
                <div
                    style={{
                        color:'#495057',
                        display: 'block',
                        textAlign:'center',
                        marginBottom:10
                    }}
                >
                    <span 
                        style={{
                            fontSize: 14,
                            fontWeight: 'bold',
                            marginLeft: 5,
                        }}>
                            Health Level
                    </span>
                    <span 
                        style={{
                            fontSize: 24,
                            display: 'block',
                            fontWeight: 'bold',
                        }}
                        id={`assurance-health-percentage-${props.name}`}
                    >
                        {`${props.percentage}%`}
                    </span>
                </div>
            </Grid>
            <Grid
                item xs={12}
            >

                    <Line data={getData(props)} options={getOptions(props)}/>
            </Grid>
        </Grid>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//