//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Item from './Item';
import moment from 'moment';
import _ from 'lodash';

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    if(props.state.assurance.networkHealth.length){

        let latestNetworkHealth = props.state.assurance.networkHealth[props.state.assurance.networkHealth.length-1];
        let result = [];

        let chartData = {
            AllClients: [],
            AllNetworkDevices: [],
            Routers: [],
            Switches: []
        }

        let keyIndex = 0;

        _.forEach(props.state.assurance.networkHealth, (health)=>{
            let xValue = moment(new Date(health.timestamp));
            chartData.AllClients.push({
                x: xValue,
                y: health.clients.totalPercentage
            });

            chartData.AllNetworkDevices.push({
                x: xValue,
                y: health.networkDevices.totalPercentage
            });

            _.forEach(health.networkDevices.networkDevices, (device)=> {
                chartData[device.deviceType].push({
                    x: xValue,
                    y: parseInt(device.healthyPercentage)
                })
            });

        });
        
        let routerCount = _.countBy(props.state.inventory, (device)=>{
            return device.type==='Router';
        });

        result.push(
            <div
                key={keyIndex++}
                style={{
                    marginBottom:40
                }}
            >
                <Item
                    name={'Routers'}
                    percentage={_.find(latestNetworkHealth.networkDevices.networkDevices, {deviceType:'Routers'}).healthyPercentage}
                    total={`${routerCount.true?routerCount.true:0}`}
                    data={chartData.Routers}
                />
            </div>
        );

        let switchCount = _.countBy(props.state.inventory, (device)=>{
            return (device.type==='Switch' || device.type==='MultiLayerSwitch');
        });

        result.push(
            <div
                key={keyIndex++}
                style={{
                    marginBottom:40
                }}
            >
                <Item
                    name={'Switches'}
                    percentage={_.find(latestNetworkHealth.networkDevices.networkDevices, {deviceType:'Switches'}).healthyPercentage}
                    total={`${switchCount.true?switchCount.true:0}`}
                    data={chartData.Switches}
                />
            </div>
        );

        result.push(
                <Item
                    key={keyIndex++}
                    name={'Hosts'}
                    percentage={latestNetworkHealth.clients.totalPercentage}
                    total={`${props.state.hosts.length}`}
                    data={chartData.AllClients}
                />
        );

        return result;
    }
    else{
        return null
    }
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//