// Import
//---------------------------------------------------------------------------//

import React, { useEffect } from 'react';
import Grid from '@material-ui/core/Grid';
import { connect } from 'react-redux';
import Container from '@material-ui/core/Container';
import Card from '@material-ui/core/Card';
import CardHeader from '@material-ui/core/CardHeader';
import CardContent from '@material-ui/core/CardContent';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faHeartbeat, faStethoscope, faUserMd} from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import AppBar from '@material-ui/core/AppBar';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import { useHistory } from 'react-router-dom';
import Assurance from './Assurance';
import Health from './Health';
import Issues from './Issues';
import CommonApi from '../../Api';

//---------------------------------------------------------------------------//

library.add(faHeartbeat, faStethoscope, faUserMd);

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    let history = useHistory();
    const navigate = (view) => {
        history.push('/' + view);
    };
    
    useEffect(() => {

        const fetchData = () =>{
            props.dispatchUtil.getNetworkHealth();
            props.dispatchUtil.getAllNetworkIssues();
            props.dispatchUtil.getNetworkDevices();
            props.dispatchUtil.getHosts();
        };

        fetchData();

        let timer = setInterval(()=>{
            fetchData();
        }, 15000);
        
        return () => {
            // Clean up the timer.
            clearTimeout(timer);
        };
    // eslint-disable-next-line
    },[]); 

    return (
        <>
            <AppBar
                position='static'
                elevation={0}
            >
                <Container maxWidth='xl'>
                    <Tabs value={0} >
                        <Tab id='navbar-tab-assurance' onClick={navigate.bind(this, 'assurance')} label='Assurance' />
                        <Tab id='navbar-tab-host' onClick={navigate.bind(this, 'host')} label='Hosts' />
                        <Tab id='navbar-tab-topology' onClick={navigate.bind(this, 'topology')} label='Topology' />
                        <Tab id='navbar-tab-pathtrace' onClick={navigate.bind(this, 'pathtrace')}label='Path Trace' />
                    </Tabs>
                </Container>
            </AppBar>
            <Container 
                maxWidth='xl'
            >
                <Grid 
                    container
                    direction='row'
                    justify='flex-start'
                    alignItems='flex-start'
                    spacing={2}
                >
                    <Grid item lg={4} md={6}>
                        <Card>
                            <CardHeader
                                avatar={
                                    <FontAwesomeIcon icon='stethoscope' size='lg' fixedWidth />
                                }
                                title='Assurance'
                                style={{
                                    borderBottom: '1px #dde0e6 solid',
                                    color: '#495057'
                                }}
                            />
                            <CardContent>
                                <Assurance {...props} />
                            </CardContent>
                        </Card>
                    </Grid>
                    <Grid item lg={4} md={6}>
                        <Card>
                            <CardHeader
                                avatar={
                                    <FontAwesomeIcon icon='heartbeat' size='lg' fixedWidth />
                                }
                                title='Network Health'
                                style={{
                                    borderBottom: '1px #dde0e6 solid',
                                    color: '#495057'
                                }}
                            />
                            <CardContent>
                                <Health {...props} />
                            </CardContent>
                        </Card>
                    </Grid>
                    <Grid item lg={4} md={6}>
                        <Card>
                            <CardHeader
                                avatar={
                                    <FontAwesomeIcon icon='user-md' size='lg' fixedWidth />
                                }
                                title='Network Issues'
                                style={{
                                    borderBottom: '1px #dde0e6 solid',
                                    color: '#495057'
                                }}
                            />
                            <CardContent>
                                <Issues {...props} />
                            </CardContent>
                        </Card>
                    </Grid>

                </Grid>
            </Container>
        </>
    );
};

//---------------------------------------------------------------------------//
// Redux Actions
//---------------------------------------------------------------------------//

const mapStateToProps = state => ({
    state: {
        assurance: state.assurance,
        inventory: state.inventory,
        hosts: state.hosts
    }
});

const mapDispatchToProps = dispatch => ({
    dispatchUtil: {
        getNetworkHealth: () => {
            CommonApi.Assurance.getNetworkHealth(dispatch);
        },
        getAllNetworkIssues: () => {
            CommonApi.Assurance.getAllNetworkIssues(dispatch);
        },
        getNetworkIssue: (id) => {
            CommonApi.Assurance.getNetworkIssue(dispatch, id);
        },
        getNetworkDevices: () => {
            CommonApi.Inventory.getNetworkDevices(dispatch);
        },
        getHosts: (id) => {
            CommonApi.Host.getHosts(dispatch);
        }
    }
});

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default connect(
    mapStateToProps,
    mapDispatchToProps
)(Component);

//---------------------------------------------------------------------------//