//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Button from '@material-ui/core/Button';
import { faPlus } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { makeStyles } from '@material-ui/core/styles';
import TextField from '@material-ui/core/TextField';
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogTitle from '@material-ui/core/DialogTitle';
import FormControl from '@material-ui/core/FormControl';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import IconButton from '@material-ui/core/IconButton';
import Input from '@material-ui/core/Input';
import InputLabel from '@material-ui/core/InputLabel';
import InputAdornment from '@material-ui/core/InputAdornment';
import Visibility from '@material-ui/icons/Visibility';
import VisibilityOff from '@material-ui/icons/VisibilityOff';

//---------------------------------------------------------------------------//

library.add(faPlus);

const useStyles = makeStyles(theme => ({
    button: {
        position: 'relative',
        top: 10,
        right: 5
    },
}));

const initState = {
    username: '',
    password: '',
    showPassword: false,
    enablePassword: '',
    showEnablePassword: false,
    description: ''
};

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const classes = useStyles();
    const [open, setOpen] = React.useState(false);

    const [state, setState] = React.useState(initState);

    const handleClickOpen = () => {
        setState(initState);
        setOpen(true);
    };

    const handleClose = () => {
        setOpen(false);
    };

    const handleChange = (event, key) => {
        let result = { ...state };
        let value = event.target.value;
        result[key] = value;
        setState(result);
    };

    const handleSubmit = () => {
        let payload = {
            username: state.username,
            password: state.password,
            enablePassword: state.enablePassword,
            description: state.description
        }

        props.dispatchUtil.addCliCredential(payload);
        setState(initState);
        setOpen(false);
    };
    
    const handleClickShowPassword = () => {
        setState({ ...state, showPassword: !state.showPassword });
    };

    const handleClickShowEnablePassword = () => {
        setState({ ...state, showEnablePassword: !state.showEnablePassword });
    };

    return (
        <>
            <Button    
                id='credential-add-button'
                className={classes.button} 
                variant='contained' 
                color='primary' 
                size='small'          
                onClick={handleClickOpen}       
                style={{
                    position: 'relative',
                    top: 5
                }}
            >
                <FontAwesomeIcon icon='plus' size='1x' fixedWidth />
                <span style={{marginLeft:5}}>
                    Credential
                </span>
            </Button>
            <Dialog open={open} onClose={handleClose} aria-labelledby='form-dialog-title'>
                <DialogTitle>New Credential</DialogTitle>
                <DialogContent>
                    <form>
                        <div>
                            <TextField 
                                id='user-cli-username' 
                                label='Username' 
                                fullWidth
                                value={state.username}
                                onChange={event => handleChange(event, 'username')}
                            />
                        </div>
                        <div>
                            <FormControl fullWidth>
                                <InputLabel htmlFor="user-cli-Password">Password</InputLabel>
                                <Input
                                    id="user-cli-Password"
                                    type={state.showPassword ? 'text' : 'password'}
                                    value={state.password}
                                    onChange={event => handleChange(event, 'password')}
                                    endAdornment={
                                        <InputAdornment position="end">
                                            <IconButton
                                                aria-label="toggle password visibility"
                                                onClick={handleClickShowPassword}
                                            >
                                            {state.showPassword ? <Visibility /> : <VisibilityOff />}
                                            </IconButton>
                                        </InputAdornment>
                                    }
                                />
                            </FormControl>
                        </div>
                        <div>
                            <FormControl fullWidth>
                                <InputLabel htmlFor="user-cli-enablePassword">Enable Password</InputLabel>
                                <Input
                                    id="user-cli-enablePassword"
                                    type={state.showEnablePassword ? 'text' : 'password'}
                                    value={state.enablePassword}
                                    onChange={event => handleChange(event, 'enablePassword')}
                                    endAdornment={
                                        <InputAdornment position="end">
                                            <IconButton
                                                aria-label="toggle password visibility"
                                                onClick={handleClickShowEnablePassword}
                                            >
                                            {state.showEnablePassword ? <Visibility /> : <VisibilityOff />}
                                            </IconButton>
                                        </InputAdornment>
                                    }
                                />
                            </FormControl>
                        </div>
                        <div>
                            <TextField 
                                id='user-cli-description' 
                                label='Description' 
                                fullWidth
                                value={state.description}
                                onChange={event => handleChange(event, 'description')}
                                type={'text'}
                            />
                        </div>
                    </form>
                </DialogContent>
                <DialogActions>
                <Button id='credential-add-cancel' onClick={handleClose} color='default'>
                    Cancel
                </Button>
                <Button id='credential-add-submit' onClick={handleSubmit} color='primary'>
                    Okay
                </Button>
                </DialogActions>
            </Dialog>
        </>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//