//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React, { useEffect } from 'react';
import Container from '@material-ui/core/Container';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import Card from '@material-ui/core/Card';
import CardHeader from '@material-ui/core/CardHeader';
import CardContent from '@material-ui/core/CardContent';
import { connect } from 'react-redux';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faUserLock } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import AppBar from '@material-ui/core/AppBar';
import CommonApi from '../../Api';
import { useHistory } from 'react-router-dom';
import Cli from './components/Cli';
import Add from './components/Add';

//---------------------------------------------------------------------------//

library.add(faUserLock);

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    let history = useHistory();

    const navigate = (view) => {
        history.push('/' + view);
    };
    useEffect(() => {
        props.dispatchUtil.fetchCliCredentials();
    },[props.dispatchUtil]); 

    return (
        <>
            <AppBar
                position='static'
                elevation={0} 
            >
                <Container maxWidth='xl'>
                    <Tabs value={2} >
                        <Tab id='navbar-tab-inventory' onClick={navigate.bind(this, 'inventory')} label='Network Device' />
                        <Tab id='navbar-tab-discovery' onClick={navigate.bind(this, 'discovery')} label='Discovery' />
                        <Tab id='navbar-tab-credential' onClick={navigate.bind(this, 'credential')}label='Credentials' />
                    </Tabs>
                </Container>
            </AppBar>
            <Container maxWidth='md'>
                <Card>
                    <CardHeader 
                        avatar={
                            <FontAwesomeIcon icon='user-lock' size='lg' fixedWidth />
                        }
                        title='CLI Credentials'
                        action={
                            <Add {...props} />
                        }
                        style={{
                            borderBottom: '1px #dde0e6 solid',
                            color: '#495057'
                        }}
                    >
                    </CardHeader>
                    <CardContent>
                        <Cli {...props} />
                    </CardContent>
                </Card>
            </Container>
        </>
    );
}

//---------------------------------------------------------------------------//
// Redux Actions
//---------------------------------------------------------------------------//

const mapStateToProps = state => ({
    state: {
        app: state.app,
        users: state.users,
        clis: state.clis
    }
});

const mapDispatchToProps = dispatch => ({
    dispatchUtil: {
        fetchCliCredentials: () => {
            CommonApi.Cli.getCliCredential(dispatch);
        },
        addCliCredential: (payload) => {
            CommonApi.Cli.addCliCredential(dispatch, payload);
        },
        removeCliCredential: (id) => {
            CommonApi.Cli.deleteGlobalCredential(dispatch, id);
        }
    }
});

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default connect(
    mapStateToProps,
    mapDispatchToProps
)(Component);

//---------------------------------------------------------------------------//