//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Grid from '@material-ui/core/Grid';
import { Line } from 'react-chartjs-2';

//---------------------------------------------------------------------------//

const FontColor = 'white';

const getData = (props) => {
    return {
        labels: [],
        datasets: [{
            data: props.data,
            label: 'Point',
            type: 'line',
            pointRadius: 0,
            fill: true,
            borderColor : '#60f090 ',
            backgroundColor: "rgba(92,240,144,.4)",
            borderWidth: 1
        }]
    };
};

const getOptions = (props) => {
    
    return {
        maintainAspectRatio: false,
        responsive: true,
        legend: {
            display: false,
        },
        scales: {
            xAxes: [{
                type: 'time',
                ticks: {
                    display: true,
                    maxTicksLimit: 5,
                    fontColor: FontColor,
                },
                gridLines: {
                    display: false
                },
                time: {
                    displayFormats: {
                        second: 'H:mm:ss'
                    }
                }
            }],
            yAxes: [{
                gridLines: {
                    display:false,
                },
                ticks: {
                    fontColor: FontColor,
                    stepSize: 25,
                    beginAtZero: true,
                    min: 0,
                    max: 100,
                    callback: (value) => {
                        return value + '%'; 
                    }
                }
            }]
        },
        tooltips: {
            intersect: false,
            mode: 'index',
            callbacks: {
                label: function(tooltipItem, myData) {
                    return `Health Level: ${parseFloat(tooltipItem.value).toFixed(2)}%`;
                }
            }
        },

        // Chart optimizations
        elements: {
            line: {
                tension: 0, // disables bezier curves
                fill: false,
                stepped: false,
                borderDash: []
            },
            point: {
                radius: 0 // default to disabled in all datasets
            }
        },
        spanGaps: true
    }

};

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    let healthText = (data) => {
        let dateOptions = { weekday: 'long', month:'long', day: 'numeric', hour: "numeric", minute:"numeric" };
        let dataDescription = data => 
            new Intl.DateTimeFormat('default', dateOptions).format(data.x)  + ", health: " + data.y + "%";

        let text = "There is no health data available at this time.";
        if(data.length > 0){
            let atext = []
            atext.push(  "Current health: " + data[data.length -1].y + "%");

            let average = Number.parseFloat(data.reduce(((a,d)=>a+Number.parseFloat(d.y)), 0) / data.length).toFixed(1);
            atext.push( "Average health: " + average + "% ");

            let dataDesc =  "Health level chart data: ";
            let adataDesc = [];
            data.slice(-5).reduce( (a,d) => (a.push(dataDescription(d)), a), adataDesc );
            atext.push( dataDesc + adataDesc.join("; "));

            text = atext.join(". ");
        }
        return text;
    };

    return (
        <Grid 
            container
            direction='row'
            justify='center'
            alignItems='center'
            aria-label={healthText(props.data)}
            role="text"
        >
            <Grid item xs={8}>
                <div
                    style={{
                        display:'block',
                        height: 130
                    }}
                >
                    <Line data={getData(props)} options={getOptions(props)}/>
                </div>
            </Grid>
            <Grid item xs={4}>
                <div
                    style={{
                        color: FontColor,
                        display: 'block',
                        textAlign:'center',
                        marginBottom: 15
                    }}
                >
                    <span 
                        style={{
                            display:'block',
                            fontSize: 14,
                            marginBottom: 5
                        }}>
                            Health Level
                    </span>
                    <span 
                        style={{
                            fontSize: 35,
                            display: 'block',
                            fontWeight: 'bold',
                        }}
                        id={`dashboard-assurance-health-${props.index}`}
                    >
                        {`${props.percentage}%`}
                    </span>
                </div>
            </Grid>
        </Grid>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//
