//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Grid from '@material-ui/core/Grid';
import {Pie} from 'react-chartjs-2';
import 'chartjs-plugin-labels';
import _ from 'lodash';

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const options = {
    maintainAspectRatio: false,
    responsive: false,
    legend: {
        position: 'left',
        labels: {
            boxWidth: 10
        }
    },
    plugins:{
        labels: {
            precision:2,
            fontColor: '#fff',
            fontSize: 12,
            render: (args) => {
                return `${Math.floor(args.percentage)}%`;
            }
        }
    }
};

const Component = (props) => {

    let unreachableDevice = 0;
    let totalDevices = 0;
    let hosts = props.state.hosts

    if(hosts.length){
        totalDevices = hosts.length
        _.forEach(hosts, (host)=>{
            if(host.pingStatus!=='SUCCESS'){
                unreachableDevice++;
            }
        });
    }

    const data = {
        labels: [
            'Ping succeeded',
            'Ping failed'
        ],
        datasets: [{
            data: [totalDevices-unreachableDevice, unreachableDevice],
            backgroundColor: [
                '#17b341',
                '#f01650',
            ],
            hoverBackgroundColor: [
                '#17b341',
                '#f01650',
            ]
        }]
    };

    return (
        <Grid 
            container
            direction='row'
            justify='center'
            alignItems='center'
            spacing={2}
            role='text'
            aria-label={'number of all hosts: '  + hosts.length + ', number of unreachable hosts: ' + unreachableDevice }
        >
            <Grid item xs={5}>
                <div
                    style={{
                        textAlign:'center',
                        color:'#495057',
                        display:'block',
                        marginTop:3
                    }}
                >
                    <span style={{position:'relative', top: -5}}>{'Hosts: '}</span>
                    <span 
                        id='dashboard-inventory-count'
                        style={{
                            fontWeight:'bold',
                            fontSize: 30,
                            color: '#e1c147'
                        }}
                        >
                        {hosts.length}
                    </span>
                </div>
            </Grid>
            <Grid item xs={7}>
                <div
                    id='dashboard-host-pie'
                >
                    <Pie data={data} height={150} width={200} options={options}/>
                </div>
            </Grid>
        </Grid>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//
