//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React, { useEffect } from 'react';
import Grid from '@material-ui/core/Grid';
import { connect } from 'react-redux';
import Container from '@material-ui/core/Container';
import Inventory from './components/Inventory';
import Assurance from './components/Assurance';
import Host from './components/Host';
import Qos from './components/Qos';
import CommonApi from '../../Api';

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    useEffect(() => {

        const fetchData = () => {
            props.dispatchUtil.getSiteHealth();
            props.dispatchUtil.getNetworkHealth();
            props.dispatchUtil.getAllNetworkIssues();
            props.dispatchUtil.getAllNetworkIssues();
            props.dispatchUtil.getPolicyTagsCount();
            props.dispatchUtil.getHosts();
            props.dispatchUtil.getNetworkDevices();
        };

        fetchData();

        let timer = setInterval(()=>{
            fetchData();
        }, 60000);
        
        return () => {
            // Clean up the timer.
            clearTimeout(timer);
        };
    // eslint-disable-next-line
    },[]); 

    return (
        <>
        <Container maxWidth='xl' style={{marginTop:20}}
            role='contentinfo'
            aria-label='network controller dashboard'
        >
            <Grid 
                container
                direction='row'
                justify='flex-start'
                alignItems='flex-start'
                spacing={3}
            >
                <Assurance {...props} />
                <Grid item lg={4} md={6}>
                    <Qos {...props} />
                </Grid>
                <Grid item lg={4} md={6}>
                    <Host {...props} />
                </Grid>
                <Grid item lg={4} md={6}>
                    <Inventory {...props} />
                </Grid>
            </Grid>
        </Container>
        </>

    );
};

//---------------------------------------------------------------------------//
// Redux Actions
//---------------------------------------------------------------------------//

const mapStateToProps = state => ({
    state: {
        dashboard: state.dashboard,
        assurance: state.assurance,
        qos: state.qos,
        hosts: state.hosts,
        inventory: state.inventory
    }
});

const mapDispatchToProps = dispatch => ({
    dispatchUtil: {
        getSiteHealth: () => {
            CommonApi.Dashboard.getSiteHealth(dispatch);
        },
        getNetworkHealth: () => {
            CommonApi.Assurance.getNetworkHealth(dispatch);
        },
        getAllNetworkIssues: () => {
            CommonApi.Assurance.getAllNetworkIssues(dispatch);
        },
        getNetworkIssue: () => {
            CommonApi.Assurance.getNetworkIssue(dispatch);
        },
        getPolicyTagsCount: () => {
            CommonApi.Qos.getPolicyTagsCount(dispatch);
        },
        getHosts: () => {
            CommonApi.Host.getHosts(dispatch);
        },
        getNetworkDevices: () => {
            CommonApi.Inventory.getNetworkDevices(dispatch);
        }
    }
});

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default connect(
    mapStateToProps,
    mapDispatchToProps
)(Component);

//---------------------------------------------------------------------------//
