//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Button from '@material-ui/core/Button';
import { faSearchPlus } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { makeStyles } from '@material-ui/core/styles';
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogTitle from '@material-ui/core/DialogTitle';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableRow from '@material-ui/core/TableRow';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'

//---------------------------------------------------------------------------//

library.add(faSearchPlus);

const useStyles = makeStyles(theme => ({
    field: {
        marginRight: 20
    }
}));

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const classes = useStyles();
    const [open, setOpen] = React.useState(false);

    const handleClickOpen = () => {
        setOpen(true);
    };

    const handleClose = () => {
        setOpen(false);
    };
    
    const row = (label, defaultValue) => {
        let value = ' ';
        if(defaultValue){
            value = defaultValue;
        }
        return(
            <TableRow 
                hover={true}
                >
                <TableCell id={`discovery-device-detail-tablecell-label-${label}`} align='right' size='small' style={{fontWeight:'bold'}}>{label}</TableCell>
                <TableCell id={`discovery-device-detail-tablecell-value-${label}`} align='left' size='small'>{value}</TableCell>
            </TableRow>
        );
    };

    return (
        <>
            <button
                aria-label={'Click for more detail'}
                id='discovery-device-detail-button'
                onClick={handleClickOpen}   
                style={{
                    width: 35,
                    height: 25,
                    borderRadius: 5
                }}
            >
                <FontAwesomeIcon icon='search-plus' size='1x' fixedWidth />
            </button>
            <Dialog open={open} onClose={handleClose} maxWidth='md' scroll='body' >
                <DialogTitle>Device Detail</DialogTitle>
                <DialogContent>
                    <Table className={classes.table}>
                        <TableBody>
                            {row('CLI Status', props.device.cliStatus)}
                            {row('Hostname', props.device.hostname)}
                            {row('ID', props.device.id)}
                            {row('Interface Count', props.device.interfaceCount)}
                            {row('Mac Address', props.device.macAddress)}
                            {row('Management Ip Address', props.device.managementIpAddress)}
                            {row('Ping Status', props.device.pingStatus)}
                            {row('Reachability Failure Reason', props.device.reachabilityFailureReason)}
                            {row('Reachability Status', props.device.reachabilityStatus)}
                            {row('Serial Number', props.device.serialNumber)}
                            {row('Type', props.device.type)}
                            {row('Up Time', props.device.upTime)}
                        </TableBody>
                    </Table>
                </DialogContent>
                <DialogActions>
                    <Button id='discovery-device-detail-cancel' onClick={handleClose} color='default'>
                        Cancel
                    </Button>
                </DialogActions>
            </Dialog>
        </>
    );
}

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//