//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableHead from '@material-ui/core/TableHead';
import TableRow from '@material-ui/core/TableRow';
import { faCog } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import _ from 'lodash';
import { makeStyles } from '@material-ui/core/styles';
import Info from './Info';

//---------------------------------------------------------------------------//

library.add(faCog);

const useStyles = makeStyles({
    table: {
        minWidth: 650,
    },
});

const Items = (props) => {
    const rows = [];
    _.forEach(props.items, (item, key) => {
        rows.push(
            <TableRow 
                key={key}
                hover={true}
            >
                <TableCell align='center' size='small'>
                    {
                        <Info device={item} />
                    }
                </TableCell>
                <TableCell size='small' id={`inventory-tablecell-hostname-${key}`}>{item.hostname}</TableCell>
                <TableCell size='small' id={`inventory-tablecell-type-${key}`}>{item.type}</TableCell>
                <TableCell size='small' id={`inventory-tablecell-managementIpAddress-${key}`}>{item.managementIpAddress}</TableCell>
                <TableCell size='small' id={`inventory-tablecell-reachabilityStatus-${key}`}>{item.reachabilityStatus}</TableCell>
            </TableRow>
        )
    });
    return rows
};

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    const classes = useStyles();
    return (
        <div
            style={{
                marginTop:30,
            }}
        >
            <hr />
            <h5>Discovered Devices</h5>
            <div
                style={{
                    overflow:'auto'
                }}
            >
                <Table className={classes.table} >
                    <TableHead>
                        <TableRow>
                            <TableCell></TableCell>
                            <TableCell size='small' style={{ fontWeight:'bold' }}>Hostname</TableCell>
                            <TableCell size='small' style={{ fontWeight:'bold' }}>Type</TableCell>
                            <TableCell size='small' style={{ fontWeight:'bold' }}>IP</TableCell>
                            <TableCell size='small' style={{ fontWeight:'bold' }}>Reachability Status</TableCell>
                        </TableRow>
                    </TableHead>
                    <TableBody>
                        { <Items items={props.state.discoveredDevices} dispatchUtil={props.dispatchUtil}/> }
                    </TableBody>
                </Table>
            </div>
        </div>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//

