//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Button from '@material-ui/core/Button';
import { faPlus } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { makeStyles } from '@material-ui/core/styles';
import TextField from '@material-ui/core/TextField';
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogTitle from '@material-ui/core/DialogTitle';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import Grid from '@material-ui/core/Grid';
import Select from 'react-select'
import _ from 'lodash';
import Toast from '../../../../components/Toast';

//---------------------------------------------------------------------------//

library.add(faPlus);

const useStyles = makeStyles(theme => ({
    field: {
        marginRight: 20
    }
}));

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    let editDiscovery = _.find(props.state.list, {id:props.discoveryId});

    const classes = useStyles();
    const [open, setOpen] = React.useState(false);
    const [discovery, setState] = React.useState({...editDiscovery, discoveryStatus: 'edit'});

    const handleClickOpen = () => {
        setState({...editDiscovery, discoveryStatus: 'edit'});
        setOpen(true);
    };

    const handleClose = () => {
        setOpen(false);
    };
    
    const handleDelete = () => {
        props.dispatchUtil.remove(discovery.id)
        setOpen(false);
    };

    const handleSave = () => {
        if(discovery.globalCredentialIdList.length){
            props.dispatchUtil.edit(discovery)
            setOpen(false);
        }else{
            Toast.error('CLI Credential is required');
        }
    };

    const handleChange = (event, key) => {
        let settings = { ...discovery };
        let value = event.target.value;
        settings[key] = value;
        setState(settings);
    };
    
    const row = (label, key) => {

        let value = discovery[key];

        if(value===undefined){
            value= '';
        }

        return(
            <TextField
                id={`discovery-edit-form-${key}`}
                margin='dense'
                label={label}
                type='text'
                variant="outlined"
                size="small"
                className={classes.field}
                value={value}
                fullWidth
                onChange={event => handleChange(event, key)}
            />
        );
    };

    const getCliOptions = () => {
        let result = [];
        _.forEach(props.clis, (cli)=>{
            result.push({
                label: `${cli.username} - ${cli.description}`,
                value: cli.id
            });
        });
        return result;
    };

    const defaultCliOptions = () => {
        let result = [];
        _.forEach(discovery.globalCredentialIdList, (id)=>{
            let cli = _.find(props.clis, {id:id})
            if(cli){
                result.push({
                    label: `${cli.username} - ${cli.description}`,
                    value: cli.id
                });
            }
        });
        return result;
    }

    return (
        <div
            style={{
                display: 'inline',
                marginLeft: 5
            }}
        >
            <Button
                id='discovery-edit-button'
                className={classes.button} 
                variant='contained' 
                color='default' 
                size='small'
                onClick={handleClickOpen}
                style={{
                    position: 'relative',
                    top: 5
                }}
            >
                <FontAwesomeIcon icon='cog' size='1x' fixedWidth />
                <span style={{marginLeft:5}}>
                    Edit
                </span>
            </Button>
            <Dialog open={open} onClose={handleClose} aria-labelledby='form-dialog-title' maxWidth='xs' scroll='paper' >
                <DialogTitle>Edit Discovery</DialogTitle>

                <DialogContent>
                    <Grid 
                        container
                        direction='row'
                        justify='flex-start'
                        alignItems='flex-start'
                        spacing={2}
                        style={{
                            marginBottom: 10
                        }}
                    >
                        <Grid item xs={5}>
                            <div
                                style={{
                                    position: 'relative',
                                    top: 10,
                                    width: 70,
                                    whiteSpace:'nowrap'
                                }}
                            >
                                <span>
                                    Discovery Type
                                </span>
                            </div>
                        </Grid>
                    </Grid>
                    <form>
                        {row('Discovery Type', 'discoveryType')}
                        {row('Name', 'name')}
                        {row('IP Address List', 'ipAddressList')}
                        {row('Timeout', 'timeOut')}
                        {row('Retry', 'retryCount')}
                        {row('CDP Level', 'cdpLevel')}
                    </form>

                    <div
                        style={{
                            marginTop: 10
                        }}
                    >
                        <Select 
                            options={getCliOptions()} 
                            closeMenuOnSelect={false}
                            isMulti
                            menuPlacement={'top'}
                            styles={{
                                container: (base) => ({ ...base, zIndex: 500 })
                            }}
                            placeholder="CLI Credential List"
                            onChange={(option)=>{
                                let settings = { ...discovery };
                                let clis = [];
                                _.forEach(option, (cli)=>{
                                    clis.push(cli.value)
                                });
                                settings['globalCredentialIdList'] = clis;
                                setState(settings);
                            }}
                            defaultValue={defaultCliOptions()}
                        />
                    </div>
                </DialogContent>
                <DialogActions>
                    <Button
                        id='discovery-edit-delete'
                        onClick={()=>{
                            handleDelete();
                        }}
                        variant='outlined'
                        color='secondary'
                        style={{
                            marginRight: 'auto'
                        }}
                    >
                        Delete
                    </Button>
                    <Button id='discovery-edit-cancel' onClick={handleClose} color='default'>
                        Cancel
                    </Button>
                    <Button id='discovery-edit-submit' onClick={handleSave} color='primary'>
                        Apply
                    </Button>
                </DialogActions>
            </Dialog>
        </div>
    );
}

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//