//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React, { useEffect } from 'react';
import { makeStyles } from '@material-ui/core/styles';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableHead from '@material-ui/core/TableHead';
import TableRow from '@material-ui/core/TableRow';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faMapMarkerAlt, faCheck, faPlay, faSatelliteDish, faSync, faStop, faClock } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import Button from '@material-ui/core/Button';
import Card from '@material-ui/core/Card';
import CardHeader from '@material-ui/core/CardHeader';
import CardContent from '@material-ui/core/CardContent';
import _ from 'lodash';
import Edit from './Edit';
import Device from './Device';

//---------------------------------------------------------------------------//

library.add(faMapMarkerAlt, faCheck, faPlay, faSatelliteDish, faSync, faStop, faClock);

const useStyles = makeStyles({
    button: {
        position: 'relative',
        top: 5,
        marginRight: '5px'
    },
    tableCell:{
        verticalAlign:'top'
    }
});

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const classes = useStyles();

    // Fetch for discovered devices if discovery is still 'In Progress'.
    useEffect(() => {
        let timer = setInterval(()=>{
            if(props.discovery.discoveryCondition === 'In Progress'){
                props.dispatchUtil.getNetworkDeviceByDiscoveryId(props.discovery.id);
            }
        }, 5000);
        
        return () => {
            // Clean up the timer.
            clearTimeout(timer);
        };
        // eslint-disable-next-line
    },[props.discovery.discoveryCondition]); 

    const conditionIcon = (condition) => {
        if(condition === 'Complete'){
            return <FontAwesomeIcon icon='check' size='lg' fixedWidth style={{color:'green'}}/>;
        }
        
        else if(condition === 'In Progress'){
            return <FontAwesomeIcon icon='sync' size='lg' spin fixedWidth style={{color:'#ffb53e'}}/>;
        }

        if(condition === 'Incomplete'){
            return <FontAwesomeIcon icon='times' size='lg' fixedWidth style={{color:'#ff6384'}}/>;
        }
        

        if(condition === 'Inactive'){
            return <FontAwesomeIcon icon='clock' size='lg' fixedWidth style={{color:'#ffb53e'}}/>;
        }

        else{
            return <FontAwesomeIcon icon='times' size='lg' fixedWidth style={{color:'#ff6384'}}/>;
        }
    };

    const credentialRow = () => {
        let result = [];
        _.forEach(props.discovery.globalCredentialIdList, (id, index)=>{
            let cli = _.find(props.clis, {id:id})
            if(cli){
                result.push(
                    <TableRow key={id}>
                        <TableCell id={`discovery-detail-tablecell-id-${index}`}  align='left' size='small'>
                            {cli.id}
                        </TableCell>
                        <TableCell id={`discovery-detail-tablecell-username-${index}`}  align='left' size='small'>
                            {cli.username}
                        </TableCell>
                        <TableCell id={`discovery-detail-tablecell-description-${index}`}  align='left' size='small'>
                            {cli.description}
                        </TableCell>
                    </TableRow>
                );
            }

        });
        return result;
    };
    
    const start = () => {
        props.dispatchUtil.start(props.discovery);
    };

    const stop = () => {
        props.dispatchUtil.stop(props.discovery);
    };

    const actionButtons = () => {

        if(props.discovery.discoveryCondition !== 'Complete') {
            return (
                <Button
                    id='discovery-detail-stop'
                    className={classes.button} 
                    variant='contained' 
                    size='small'
                    onClick={stop}
                    style={{
                        background: 'red',
                        color: 'white'
                    }}
                >
                    <FontAwesomeIcon icon='stop' size='1x' fixedWidth />
                    <span style={{marginLeft:5}}>
                        Stop
                    </span>
                </Button>
            );
        }else{
            return (<>
                <Edit {...props} discoveryId={props.discovery.id} />
                <Button   
                    id='discovery-detail-start'
                    className={classes.button} 
                    variant='contained' 
                    color='primary' 
                    size='small'          
                    onClick={start}       
                    style={{
                        position: 'relative',
                        top: 5,
                        marginLeft: 8
                    }}
                >
                    <FontAwesomeIcon icon='play' size='1x' fixedWidth />
                    <span style={{marginLeft:5}}>
                        Start
                    </span>
                </Button>
            </>);
        }
    };

    const getIpAddressList = () =>{
        let result = []
        _.forEach(props.discovery.ipAddressList.split(','), (ip, index)=>{
            result.push(
                <span
                    key={index}
                    style={{
                        display:'block'
                    }}
                >
                    {ip}
                </span>
            )
        })
        return result;
    };

    return (
        <Card>
            <CardHeader
                avatar={
                    <FontAwesomeIcon icon='map-marker-alt' size='lg' fixedWidth />
                }
                title={props.discovery.name}
                action={
                    actionButtons()
                }
                style={{
                    borderBottom: '1px #dde0e6 solid',
                    color: '#495057'
                }}
            />
            <CardContent>
                <div
                    id='discovery-detail-discoveryCondition'
                    style={{
                        marginBottom: 5
                    }}
                >
                    <b>{'Condition: '}</b>
                    <span>
                        {props.discovery.discoveryCondition}
                    </span>
                    <div
                        style={{
                            marginLeft: 10,
                            display: 'inline'
                        }}
                    >
                        {conditionIcon(props.discovery.discoveryCondition)}
                    </div>
                </div>
                <div
                    id='discovery-detail-discoveryStatus'
                    style={{
                        marginBottom: 5
                    }}
                >
                    <b>{'Status: '}</b>
                    <span>
                        {props.discovery.discoveryStatus}
                    </span>
                </div>
                <div
                    id='discovery-detail-discoveryType'
                    style={{
                        marginBottom: 5
                    }}
                >
                    <b>{'Type: '}</b>
                    <span>
                        {props.discovery.discoveryType}
                    </span>
                </div>
                <div
                    id='discovery-detail-id'
                    style={{
                        marginBottom: 5
                    }}
                >
                    <b>{'ID: '}</b>
                    <span>
                        {props.discovery.id}
                    </span>
                </div>
                <hr />

                <h5>
                    Discovery Details
                </h5>

                <Table>
                    <TableHead>
                        <TableRow>
                            <TableCell size='small' style={{ fontWeight: 'bold' }}>CDP Level</TableCell>
                            <TableCell size='small' style={{ fontWeight: 'bold' }}>Retry Count</TableCell>
                            <TableCell size='small' style={{ fontWeight: 'bold' }}>TimeOut</TableCell>
                            <TableCell size='small' style={{ fontWeight: 'bold' }}>IP Range</TableCell>
                        </TableRow>
                    </TableHead>
                    <TableBody>
                        <TableRow>
                            <TableCell 
                                id='discovery-detail-cdpLevel' 
                                size='small'
                                className={classes.tableCell}
                            > 
                                {props.discovery.cdpLevel} 
                            </TableCell>
                            <TableCell 
                                id='discovery-detail-retryCount' 
                                className={classes.tableCell} 
                                size='small'
                            >
                                {props.discovery.retryCount}
                            </TableCell>
                            <TableCell 
                                id='discovery-detail-timeOut' 
                                className={classes.tableCell} 
                                size='small'
                            >
                                {props.discovery.timeOut}
                            </TableCell>
                            <TableCell 
                                id='discovery-detail-ipAddressList' 
                                className={classes.tableCell} 
                                size='small'
                            >
                                {getIpAddressList()}
                            </TableCell>
                        </TableRow>
                    </TableBody>
                </Table>

                <hr />

                <h5>
                    CLI Credentials
                </h5>

                <Table>
                    <TableHead>
                        <TableRow>
                            <TableCell size='small' style={{ fontWeight: 'bold' }}>ID</TableCell>
                            <TableCell size='small' style={{ fontWeight: 'bold' }}>Username</TableCell>
                            <TableCell size='small' style={{ fontWeight: 'bold' }}>Description</TableCell>
                        </TableRow>
                    </TableHead>
                    <TableBody>
                        {credentialRow()}
                    </TableBody>
                </Table>  
                <Device {...props} />
            </CardContent>
        </Card>

    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//