//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Button from '@material-ui/core/Button';
import { faPlus } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { makeStyles } from '@material-ui/core/styles';
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogTitle from '@material-ui/core/DialogTitle';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import Grid from '@material-ui/core/Grid';
import Select from 'react-select';
import _ from 'lodash';
import CDP from './cdp';
import Range from './range';
import Toast from '../../../../../components/Toast';

//---------------------------------------------------------------------------//

library.add(faPlus);

const useStyles = makeStyles(theme => ({
    root: {
        overflow: 'visible'
    },
    field: {
        marginRight: 20
    }
}));

const defaultState = {
    discoveryType: 'CDP',
    name: '',
    ipAddressList: '',
    cdpLevel: 16,
    timeout: 5,
    retry: 3,
    globalCredentialIdList: []
};

const DiscoveryTypeOptions = [
    { value: 'CDP', label: 'CDP' },
    { value: 'Range', label: 'Range' }
]

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const classes = useStyles();
    const [open, setOpen] = React.useState(false);
    const [discovery, setState] = React.useState(defaultState);

    const handleClickOpen = () => {
        setOpen(true);
        setState(defaultState);
    };

    const payloadChange = (value, key) => {
        let settings = { ...discovery };
        settings[key] = value;
        setState(settings);
    };

    const handleClose = () => {
        setOpen(false);
    };
    
    const handleAdd = () => {
        if(discovery.globalCredentialIdList.length){
            props.dispatchUtil.add(discovery)
            setOpen(false);
        }else{
            Toast.error('CLI Credential is required');
        }
    };
    
    const getCliOptions = () => {
        let result = _.map(props.clis, (cli)=>{
            return {
                value: cli.id,
                label: `${cli.username} - ${cli.description}`
            }
        });
        return result;
    };

    const getForm = () => {
        if(discovery.discoveryType==='CDP'){
            return <CDP payloadChange={payloadChange} />
        }else{
            return <Range payloadChange={payloadChange} />
        }
    };

    return (
        <div
            style={{
                display: 'inline',
                marginLeft: 5
            }}
        >
            <Button    
                aria-label={"Click to add new discovery"}
                className={classes.button} 
                variant='contained' 
                color='primary' 
                size='small'          
                onClick={handleClickOpen}       
                style={{
                    position: 'relative',
                    top: 5
                }}
            >
                <FontAwesomeIcon icon='plus' size='1x' fixedWidth />
                <span id='discovery-add-button' style={{marginLeft:5}}>
                    Discovery
                </span>
            </Button>
            <Dialog 
                open={open} 
                onClose={handleClose} 
                aria-labelledby='discovery-dialog-add' 
                maxWidth='xs' 
                scroll='body'
                classes={{ 
                    paperScrollBody: classes.root 
                }}
            >
                <DialogTitle>New Discovery</DialogTitle>
                <DialogContent>
                    <Grid 
                        container
                        direction='row'
                        justify='flex-start'
                        alignItems='flex-start'
                        spacing={2}
                        style={{
                            marginBottom: 10
                        }}
                    >
                        <Grid item xs={5}>
                            <div
                                style={{
                                    position: 'relative',
                                    top: 10,
                                    width: 70,
                                    whiteSpace:'nowrap'
                                }}
                            >
                                <span>
                                    Discovery Type
                                </span>
                            </div>
                        </Grid>
                        <Grid item xs={7}>
                            <div
                                style={{
                                    position: 'relative',
                                    top: 2,
                                    width: '100%',
                                    float: 'right'
                                }}
                            >
                                <Select
                                    aria-label="Click to select discovery"
                                    id='discovery-add-type-select'
                                    classNamePrefix='discovery-add-type-select'
                                    options={DiscoveryTypeOptions}
                                    defaultValue={[
                                        { value: 'CDP', label: 'CDP', isFixed: true }
                                    ]} 
                                    styles={{
                                        container: (base) => ({ ...base, zIndex: 500 })
                                    }}
                                    onChange={(option)=>{
                                        let settings = { ...discovery };
                                        let value = option.value;
                                        settings['discoveryType'] = value;
                                        setState(settings);
                                    }}
                                />
                            </div>
                        </Grid>
                    </Grid>

                    {getForm()}

                    <div
                        style={{
                            marginTop: 10
                        }}
                    >
                        <Select 
                            id='discovery-add-cli-select'
                            classNamePrefix='discovery-add-cli-select'
                            options={getCliOptions()} 
                            closeMenuOnSelect={false}
                            isMulti
                            menuPlacement={'top'}
                            styles={{
                                container: (base) => ({ ...base, zIndex: 500 })
                            }}
                            placeholder="CLI Credential List"
                            onChange={(option)=>{
                                let settings = { ...discovery };
                                let clis = [];
                                _.forEach(option, (cli)=>{
                                    clis.push(cli.value)
                                });
                                settings['globalCredentialIdList'] = clis;
                                setState(settings);
                            }}
                        />
                    </div>

                </DialogContent>
                <DialogActions>
                    <Button id='discovery-add-cancel' onClick={handleClose} color='default'>
                        Cancel
                    </Button>
                    <Button id='discovery-add-submit' onClick={handleAdd} color='primary'>
                        Add
                    </Button>
                </DialogActions>
            </Dialog>
        </div>
    );
}

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//