//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React, { useEffect } from 'react';
import { makeStyles } from '@material-ui/core/styles';
import TextField from '@material-ui/core/TextField';

//---------------------------------------------------------------------------//

const useStyles = makeStyles(theme => ({
    root: {
        overflow: 'visible'
    },
    field: {
        marginRight: 20
    }
}));

const defaultState = {
    name: '',
    startIpAddress: '',
    endIpAddress: ''
};

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const classes = useStyles();
    const [form, setState] = React.useState(defaultState);

    const handleNameChange = (event, key) => {
        let settings = { ...form };
        let value = event.target.value;
        settings[key] = value;
        setState(settings);
        props.payloadChange(value, key)
    };

    const handleIpChange = (event, key) => {
        let settings = { ...form };
        let value = event.target.value;
        settings[key] = value;
        setState(settings);
    };
    
    useEffect(() => {
        if(form['startIpAddress'].length && form['endIpAddress'].length){
            let ipAddressList = '';
            ipAddressList = `${form['startIpAddress']},${form['endIpAddress']}`;
            props.payloadChange(ipAddressList, 'ipAddressList')
        }
        // eslint-disable-next-line
    },[form]); 
    
    const row = (label, key) => {

        let value = form[key];

        if(value===undefined){
            value= '';
        }

        return(
            <TextField
                id={`discovery-add-range-${key}`}
                margin='dense'
                label={label}
                type='text'
                variant="outlined"
                size="small"
                className={classes.field}
                value={value}
                fullWidth
                onChange={event => handleIpChange(event, key)}
            />
        );
    };

    return (
        <form>
            <TextField
                id='discovery-add-range-name'
                margin='dense'
                label='Name'
                type='text'
                variant="outlined"
                size="small"
                className={classes.field}
                value={form.name}
                fullWidth
                onChange={event => handleNameChange(event, 'name')}
            />
            {row('Start IP Address', 'startIpAddress')}
            {row('End IP Address', 'endIpAddress')}
        </form>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//