//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableHead from '@material-ui/core/TableHead';
import TableRow from '@material-ui/core/TableRow';
import { makeStyles } from '@material-ui/core/styles';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faSatelliteDish, faCheck, faSync, faPlus, faClock } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import Card from '@material-ui/core/Card';
import CardHeader from '@material-ui/core/CardHeader';
import CardContent from '@material-ui/core/CardContent';
import _ from 'lodash';
import Add from './Add';

//---------------------------------------------------------------------------//

const useStyles = makeStyles(theme => ({
    cardContent:{
        maxHeight:400,
        overflow:'auto'
    },
    tableCell: {
        color: '#1e4471'
    },
    tableRow: {
        cursor: 'pointer'
    }
}));

library.add(faSatelliteDish, faCheck, faSync, faPlus, faClock);

const Icon = (prop) => {

    let StatusIcon = <FontAwesomeIcon icon='sync' size='lg' spin fixedWidth style={{color:'#ffb53e'}}/>;

    if(prop.status === 'Complete'){
        StatusIcon = <FontAwesomeIcon icon='check' size='lg' fixedWidth style={{color:'green'}}/>
    }else if(prop.status === 'Inactive'){
        StatusIcon = <FontAwesomeIcon icon='clock' size='lg' fixedWidth style={{color:'#ffb53e'}}/>;
    }

    return(
        <div 
            style={{
                whiteSpace:'nowrap'
            }}
        >
            {StatusIcon}
            <span
                style={{
                    marginLeft: 5
                }}
            >
                {` ${prop.status}`}
            </span>
        </div>
    );
};

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    const classes = useStyles();
    const clickedRow = (id) => {
        props.dispatchUtil.selectDiscovery(id);
        props.dispatchUtil.getNetworkDeviceByDiscoveryId(id);
    };

    return (
        <Card>
            <CardHeader
                avatar={
                    <FontAwesomeIcon icon='satellite-dish' size='lg' fixedWidth />
                }
                title='Discoveries'
                action={
                    <Add {...props} />
                }
                style={{
                    borderBottom: '1px #dde0e6 solid',
                    color: '#495057'
                }}
            />
            <CardContent className={classes.cardContent} >
                <Table>
                    <TableHead>
                        <TableRow>
                            <TableCell align='left' size='small' style={{ fontWeight: 'bold' }}>Status</TableCell>
                            <TableCell align='left' size='small' style={{ fontWeight: 'bold' }}>Name</TableCell>
                        </TableRow>
                    </TableHead>
                    <TableBody>
                        {_.map(props.state.list, (row, index) => (
                            <TableRow 
                                className={classes.tableRow} 
                                key={index}
                                hover={true}
                                selected={props.state.selectedId===row.id}
                                onClick={()=>{
                                    clickedRow(row.id);
                                }}
                            >
                                <TableCell 
                                    id={`discovery-list-tablecell-discoveryCondition-${index}`} 
                                    className={classes.tableCell} 
                                    align='left' 
                                    size='small' 
                                    padding='checkbox'
                                >
                                    <Icon status={row.discoveryCondition} />
                                </TableCell>
                                <TableCell 
                                    id={`discovery-list-tablecell-name-${index}`} 
                                    className={classes.tableCell} 
                                    align='left' 
                                    size='small'>{row.name}
                                </TableCell>
                            </TableRow>
                        ))}
                    </TableBody>
                </Table>
            </CardContent>
        </Card>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//