//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React, { useEffect } from 'react';
import Grid from '@material-ui/core/Grid';
import Container from '@material-ui/core/Container';
import AppBar from '@material-ui/core/AppBar';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import { useHistory } from 'react-router-dom';
import { connect } from 'react-redux';
import useDeepCompareEffect from 'use-deep-compare-effect';
import _ from 'lodash';
import List from './components/List';
import Detail from './components/Detail';
import CommonApi from '../../Api';

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    let history = useHistory();

    const navigate = (view) => {
        history.push('/' + view);
    };

    useEffect(() => {
        props.dispatchUtil.getDiscoveries();
        props.dispatchUtil.fetchCliCredentials();
        // eslint-disable-next-line
    },[]); 
    
    useDeepCompareEffect(() => {
        let timer = setInterval(()=>{
            let fetch = false;
            _.forEach(props.state.list, (discovery)=>{
                if(discovery.discoveryCondition !== 'Complete'){
                    fetch=true;
                }
            });
            if(fetch){
                props.dispatchUtil.getDiscoveries();
            }
        }, 5000);
        
        return () => {
            // Clean up the timer.
            clearTimeout(timer);
        };
    },[props.state.list])

    const getDetail = () => {

        if(props.state.list.length){
            
            let discovery = _.find(props.state.list, {id: props.state.selectedId});

            if(discovery===undefined){
                return null;
            }else{

                return(
                    <Grid item lg={8} md={7}>
                        <Detail clis={props.clis} state={props.state} discovery={discovery} dispatchUtil={props.dispatchUtil}/>
                    </Grid>
                );
            }
        }else{
            return null;
        }
    };

    return (
        <>
            <AppBar
                position='static'
                elevation={0} 
            >
                <Container maxWidth='xl'>
                    <Tabs value={1} >
                        <Tab id='navbar-tab-inventory' onClick={navigate.bind(this, 'inventory')} label='Network Device' />
                        <Tab id='navbar-tab-discovery' onClick={navigate.bind(this, 'discovery')}label='Discovery' />
                        <Tab id='navbar-tab-credential' onClick={navigate.bind(this, 'credential')}label='Credentials' />
                    </Tabs>
                </Container>
            </AppBar>
            <Container maxWidth='lg'>
                <Grid 
                    container
                    direction='row'
                    justify='center'
                    alignItems='flex-start'
                    spacing={2}
                >
                    <Grid item lg={4} md={5}>
                        <List {...props}/>
                    </Grid>
                    {getDetail()}
                </Grid>
            </Container>
        </>
    );
}

//---------------------------------------------------------------------------//
// Redux Actions
//---------------------------------------------------------------------------//

const mapStateToProps = state => ({
    state: state.discovery,
    clis: state.clis
});

const mapDispatchToProps = dispatch => ({
    
    dispatchUtil: {
        
        getDiscoveries: () => {
            CommonApi.Discovery.getDiscovery(dispatch);
        },

        getDiscovery: (id, showToast) => {
            CommonApi.Discovery.getDiscoveryById(dispatch, id, showToast);
        },
        
        add: (discovery) => {
            CommonApi.Discovery.insertDiscovery(dispatch, discovery);
        },

        start: (discovery) => {
            CommonApi.Discovery.start(dispatch, discovery);
        },

        stop: (discovery) => {
            CommonApi.Discovery.stop(dispatch, discovery);
        },

        edit: (discovery) => {
            CommonApi.Discovery.updateDiscovery(dispatch, discovery);
        },
        
        remove: (id) => {
            CommonApi.Discovery.deleteDiscoveryById(dispatch, id);
        },

        selectDiscovery: (id) => {
            dispatch({
                type: 'DISCOVERY_SET_SELECTED',
                payload: {
                    id: id
                }
            });
        },

        fetchCliCredentials: () => {
            CommonApi.Cli.getCliCredential(dispatch);
        },

        getNetworkDeviceByDiscoveryId: (id) => {
            CommonApi.Discovery.getNetworkDeviceByDiscoveryId(dispatch, id);
        }

    }
});

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default connect(
    mapStateToProps,
    mapDispatchToProps
)(Component);


//---------------------------------------------------------------------------//