//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Button from '@material-ui/core/Button';
import { faCog } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { makeStyles } from '@material-ui/core/styles';
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogTitle from '@material-ui/core/DialogTitle';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableRow from '@material-ui/core/TableRow';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'

//---------------------------------------------------------------------------//

library.add(faCog);

const useStyles = makeStyles(theme => ({
    field: {
        marginRight: 20
    }
}));

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const classes = useStyles();
    const [open, setOpen] = React.useState(false);

    const handleClickOpen = () => {
        setOpen(true);
    };

    const handleClose = () => {
        setOpen(false);
    };
    
    const infoRow = (label, key, setValue) => {
        let value = props.device[key];
        if(setValue){
            value = setValue;
        }
        return(
            <TableRow hover={true}>
                <TableCell id={`host-detail-tablecell-label-${label}`} align='right' size='small' style={{fontWeight:'bold'}}>{label}</TableCell>
                <TableCell id={`host-detail-tablecell-value-${label}`} align='left' size='small'>{value}</TableCell>
            </TableRow>
        );
    };

    const handleDelete = () => {
        props.dispatchUtil.deleteHostByIp(props.device.hostIp);
        setOpen(false);
    };

    return (
        <>
            <button
                aria-label={"Click for more detail"}
                id={`host-detail-button-${props.index}`}
                onClick={handleClickOpen}   
                style={{
                    width: 35,
                    height: 25,
                    borderRadius: 5
                }}
            >
                <FontAwesomeIcon icon='cog' size='1x' fixedWidth />
            </button>
            <Dialog open={open} onClose={handleClose} maxWidth='md' scroll='body' >
                <DialogTitle>Host Detail</DialogTitle>
                <DialogContent>
                    <Table className={classes.table}>
                        <TableBody>
                            {infoRow('Connected AP MAC Address', 'connectedAPMacAddress')}
                            {infoRow('Connected AP Name', 'connectedAPName')}
                            {infoRow('Connected Network Interface Name', 'connectedInterfaceName')}
                            {infoRow('Connected Network Device IP Address', 'connectedNetworkDeviceIpAddress')}
                            {infoRow('Connected Network Device Name', 'connectedNetworkDeviceName')}
                            {infoRow('Host IP', 'hostIp')}
                            {infoRow('Host MAC', 'hostMac')}
                            {infoRow('Host Name', 'hostName')}
                            {infoRow('Host Type', 'hostType')}
                            {infoRow('ID', 'id')}
                            {infoRow('Last Updated', 'lastUpdated')}
                            {infoRow('Ping Status', 'pingStatus')}
                            {infoRow('VLAN ID', 'vlanId')}
                        </TableBody>
                    </Table>
                </DialogContent>
                <DialogActions>
                    <Button
                        id='host-edit-delete'
                        onClick={handleDelete}
                        variant='outlined'
                        color='secondary'
                        style={{
                            marginRight: 'auto'
                        }}
                    >
                        Delete
                    </Button>
                    <Button id='host-detail-cancel' onClick={handleClose} color='default'>
                        Cancel
                    </Button>
                </DialogActions>
            </Dialog>
        </>
    );
}

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//