//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import { makeStyles } from '@material-ui/core/styles';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableHead from '@material-ui/core/TableHead';
import TableRow from '@material-ui/core/TableRow';
import _ from 'lodash';
import Info from '../Info';

//---------------------------------------------------------------------------//

const useStyles = makeStyles({
    table: {
        minWidth: 650,
    },
    tableCell: {
        textAlign: 'left',
        whiteSpace:'nowrap'
    }
});

const Items = (props) => {
    const rows = [];
    const classes = useStyles();
    _.forEach(props.items, (item, key) => {
        rows.push(
            <TableRow 
                key={key}
                hover={true}
            >
                <TableCell align='left' size='small'>
                    <Info device={item} {...props} index={key} />
                </TableCell>
                <TableCell className={classes.tableCell} size='small' id={`host-tablecell-hostMac-${key}`}>{item.hostMac}</TableCell>
                <TableCell className={classes.tableCell} size='small' id={`host-tablecell-hostIp-${key}`}>{item.hostIp}</TableCell>
                <TableCell className={classes.tableCell} size='small' id={`host-tablecell-hostName-${key}`}>{item.hostName}</TableCell>
                <TableCell className={classes.tableCell} size='small' id={`host-tablecell-hostType-${key}`} >{item.hostType}</TableCell>
                <TableCell className={classes.tableCell} size='small' id={`host-tablecell-connectedNetworkDeviceIpAddress-${key}`} style={{borderLeft:'grey solid 1px'}}>{item.connectedNetworkDeviceIpAddress}</TableCell>
                <TableCell className={classes.tableCell} size='small' id={`host-tablecell-connectedNetworkDeviceName-${key}`}>{item.connectedNetworkDeviceName}</TableCell>
                <TableCell className={classes.tableCell} size='small' id={`host-tablecell-connectedNetworkDeviceInterfaceName-${key}`}>{item.connectedInterfaceName}</TableCell>
            </TableRow>
        )
    });
    return rows
};

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    const classes = useStyles();
    return (
        <Table className={classes.table} aria-label='simple table'>
            <TableHead>
                <TableRow>
                    <TableCell></TableCell>
                    <TableCell colSpan={4} align='center' size='small' style={{ fontWeight:'bold', background:'#008516', color:'white' }}>Host Device</TableCell>
                    <TableCell colSpan={3} align='center' size='small' style={{ fontWeight:'bold', background:'#004BAF', color:'white' }}>Connected Network Device</TableCell>
                </TableRow>
                <TableRow>
                    <TableCell></TableCell>
                    <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}>MAC</TableCell>
                    <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}>IP</TableCell>
                    <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}>Hostname</TableCell>
                    <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}>Type</TableCell>
                    <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold', borderLeft:'grey solid 1px' }}>IP</TableCell>
                    <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}>Hostname</TableCell>
                    <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}>Port</TableCell>
                </TableRow>
            </TableHead>
            <TableBody>
                { <Items items={props.items} {...props}/> }
            </TableBody>
        </Table>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//