//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import useDeepCompareEffect from 'use-deep-compare-effect';
import Grid from '@material-ui/core/Grid';
import Container from '@material-ui/core/Container';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faDesktop } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import Card from '@material-ui/core/Card';
import CardHeader from '@material-ui/core/CardHeader';
import CardContent from '@material-ui/core/CardContent';
import AppBar from '@material-ui/core/AppBar';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import { useHistory } from 'react-router-dom';
import { connect } from 'react-redux';
import Table from './components/Table';
import CommonApi from '../../Api';

//---------------------------------------------------------------------------//

library.add(faDesktop);

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    
    let history = useHistory();
    const navigate = (view) => {
        history.push('/' + view);
    };
    
    useDeepCompareEffect(() => {
        props.dispatchUtil.fetchList();
        let timer = setInterval(()=>{
            props.dispatchUtil.fetchList();
            /*
            _.forEach(props.state, (item)=>{
                props.dispatchUtil.hostById(item.id);
            })*/
        }, 15000);
        
        return () => {
            // Clean up the timer.
            clearTimeout(timer);
        };
        // eslint-disable-next-line
    },[props.state]); 
    
    return (
        <>
            <AppBar
                position='static'
                elevation={0}   
            >
                <Container maxWidth='xl'>
                    <Tabs value={1} >
                        <Tab id='navbar-tab-assurance' onClick={navigate.bind(this, 'assurance')} label='Assurance' />
                        <Tab id='navbar-tab-host' onClick={navigate.bind(this, 'host')} label='Hosts' />
                        <Tab id='navbar-tab-topology' onClick={navigate.bind(this, 'topology')} label='Topology' />
                        <Tab id='navbar-tab-pathtrace' onClick={navigate.bind(this, 'pathtrace')}label='Path Trace' />
                    </Tabs>
                </Container>
            </AppBar>

            <Container maxWidth='lg'>
                <Grid 
                    container
                    direction='row'
                    justify='center'
                    alignItems='flex-start'
                >
                    <Card>
                        <CardHeader
                            avatar={
                                <FontAwesomeIcon icon='desktop' size='lg' fixedWidth />
                            }
                            title='Host'
                            style={{
                                borderBottom: '1px #dde0e6 solid',
                                color: '#495057'
                            }}
                        />
                        <CardContent>
                            <Table items={props.state} {...props} />
                        </CardContent>
                    </Card>
                </Grid>
            </Container>
        </>
    );
};

//---------------------------------------------------------------------------//
// Redux Actions
//---------------------------------------------------------------------------//

const mapStateToProps = state => ({
    state: state.hosts
});

const mapDispatchToProps = dispatch => ({
    dispatchUtil: {
        fetchList: () => {
            CommonApi.Host.getHosts(dispatch);
        },
        deleteHostByIp: (ip) => {
            CommonApi.Host.deleteHostByIp(dispatch, ip);
        }
    }
});

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default connect(
    mapStateToProps,
    mapDispatchToProps
)(Component);


//---------------------------------------------------------------------------//