//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Button from '@material-ui/core/Button';
import { faPlus } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { makeStyles } from '@material-ui/core/styles';
import TextField from '@material-ui/core/TextField';
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogTitle from '@material-ui/core/DialogTitle';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import Select from 'react-select';
import _ from 'lodash';

//---------------------------------------------------------------------------//

library.add(faPlus);

const useStyles = makeStyles(theme => ({
    root: { 
        overflow: 'visible'
    },
    button: {
        position: 'relative',
        top: 10,
        right: 5
    },
    field: {
        marginRight: 20
    }
}));

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const classes = useStyles();
    const [open, setOpen] = React.useState(false);
    const [device, setState] = React.useState({
        ipAddress: '',
        globalCredentialId: ''
    });

    const handleClickOpen = () => {
        setOpen(true);
    };

    const handleClose = () => {
        setOpen(false);
    };

    const handleSubmit = () => {
        props.dispatchUtil.addDevice({
            ipAddress: device.ipAddress,
            globalCredentialId: device.globalCredentialId
        });
        setOpen(false);
    };
    
    const handleChange = (event, key) => {
        let settings = { ...device };
        let value = event.target.value;
        settings[key] = value;
        setState(settings);
    };

    const field = (label, key) => {
        return(
            <TextField
                id={`inventory-add-textfield-${key}`}
                margin='dense'
                label={label}
                type='text'
                variant="outlined"
                size="small"
                className={classes.field}
                value={device[key]}
                onChange={event => handleChange(event, key)}
                style={{
                    width:'100%'
                }}
            />
        );
    };

    const getCliOptions = () => {
        let result = _.map(props.clis, (cli)=>{
            return {
                value: cli.id,
                label: `${cli.username} - ${cli.description}`
            }
        });
        return result;
    };

    return (
        <>
            <Button
                id='inventory-add-button'
                className={classes.button} 
                variant='contained' 
                color='primary' 
                size='small'          
                onClick={handleClickOpen}       
                style={{
                    position: 'relative',
                    top: 5
                }}
            >
                <FontAwesomeIcon icon='plus' size='1x' fixedWidth />
                <span style={{marginLeft:5}}>
                    Device
                </span>
            </Button>
            <Dialog 
                classes={{ paperScrollBody: classes.root }}
                open={open} 
                onClose={handleClose} 
                aria-labelledby='form-dialog-title' 
                maxWidth='xs' 
                scroll='body'
                fullWidth
            >
                <DialogTitle id='form-dialog-title'>Add New Device</DialogTitle>
                <DialogContent
                    className={classes.root}
                >

                    <form>
                        {field('IP Address', 'ipAddress')}
                    </form>

                    <Select
                        id='inventory-add-cli-select'
                        classNamePrefix='inventory-add-cli-select'
                        options={getCliOptions()} 
                        closeMenuOnSelect={true}
                        menuPlacement={'bottom'}
                        styles={{
                            container: (base) => ({
                                ...base, 
                                marginTop:10,
                                marginBottom:25
                            })
                        }}
                        placeholder='CLI Credential'
                        onChange={(option)=>{
                            let settings = { ...device };
                            settings['globalCredentialId'] = option.value;
                            setState(settings);
                        }}
                    />
                </DialogContent>
                <DialogActions>
                    <Button id='inventory-add-cancel' onClick={handleClose} color='default'>
                        Cancel
                    </Button>
                    <Button id='inventory-add-okay' onClick={handleSubmit} color='primary'>
                        Add
                    </Button>
                </DialogActions>
            </Dialog>
        </>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//