//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Button from '@material-ui/core/Button';
import { faPlus } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { makeStyles } from '@material-ui/core/styles';
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogTitle from '@material-ui/core/DialogTitle';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import Grid from '@material-ui/core/Grid';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableRow from '@material-ui/core/TableRow';
import Select from 'react-select';
import _ from 'lodash';

//---------------------------------------------------------------------------//

library.add(faPlus);

const useStyles = makeStyles(theme => ({
    root: { 
        overflow: 'visible'
    },
    field: {
        marginRight: 20
    }
}));


//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    let initialConfig = {
        ipAddress: props.device.managementIpAddress,
        globalCredentialId: props.device.globalCredentialId
    }

    const classes = useStyles();
    const [open, setOpen] = React.useState(false);
    const [device, setState] = React.useState(initialConfig);

    const handleClickOpen = () => {
        setOpen(true);
        setState(initialConfig);
    };

    const handleClose = () => {
        setOpen(false);
    };
    
    const handleDelete = (ip) => {
        props.dispatchUtil.removeDevice(ip)
        setOpen(false);
    };

    const handleSave = () => {
        props.dispatchUtil.editDevice(device)
        setOpen(false);
    };

    const infoRow = (label, key, setValue) => {
        let value = props.device[key];
        if(setValue){
            value = setValue;
        }
        
        return(
            <TableRow 
                key={key}
                hover={true}
            >
                <TableCell size='small' style={{fontWeight:'bold', textAlign:'right'}}>{label}</TableCell>
                <TableCell size='small' id={`inventory-edit-info-${key}`}>{value}</TableCell>
            </TableRow>
        );
    };

    const infoRowArray = (label, key) => {
        let values = props.device[key];
        let result = [];

        _.forEach(values, (value, index)=>{
            let spanValue = '-';
            if(value.length){
                spanValue = value;
            }
            result.push(
                <span
                    id={`inventory-edit-info-${key}-${index}`}
                    key={`${key}-${index}`}
                    style={{
                        display:'block'
                    }}
                >
                    {spanValue}
                </span>
            );
        })

        return(
            <TableRow 
                key={key}
                hover={true}
            >
                <TableCell 
                    size='small' 
                    style={{
                        fontWeight:'bold', 
                        textAlign:'right', 
                        verticalAlign:'top'
                    }}
                >
                    {label}
                </TableCell>
                <TableCell size='small'>{result}</TableCell>
            </TableRow>
        );

    };

    const getCliOptions = () => {
        let result = _.map(props.clis, (cli)=>{
            return {
                value: cli.id,
                label: `${cli.username} - ${cli.description}`
            }
        });
        return result;
    };

    const getDefaultCliOptions = (options) => {
        let result = [];
        _.forEach(getCliOptions(), (cli)=>{
            if(cli.value === device.globalCredentialId){
                result.push(cli);
            }
        });
        return result;
    };

    return (
        <>
            <button
                aria-label={"Click for more detail"}
                id='inventory-edit-button'
                onClick={handleClickOpen}   
                style={{
                    width: 35,
                    height: 25,
                    borderRadius: 5
                }}
            >
                <FontAwesomeIcon icon='cog' size='1x' fixedWidth />
            </button>
            <Dialog 
                classes={{ 
                    paperScrollBody: classes.root 
                }}
                open={open} 
                onClose={handleClose} 
                aria-labelledby='form-dialog-title' 
                maxWidth='md' 
                scroll='body' 
                fullWidth
            >
                <DialogTitle>Update Network Device</DialogTitle>
                <DialogContent
                    className={classes.root}
                >
                    <Grid 
                        container
                        direction='row'
                        justify='center'
                        alignItems='flex-start'
                        spacing={3}
                    >
                        <Grid item md={8}>
                            <h4 style={{textAlign:'center'}}>Device Detail</h4>
                            <Table>
                                <TableBody>
                                    {infoRow('Hostname', 'hostname')}
                                    {infoRow('ID', 'id')}
                                    {infoRow('Interface Count', 'interfaceCount')}
                                    {infoRow('Software Version', 'softwareVersion')}
                                    {infoRow('MAC Address', 'macAddress')}
                                    {infoRow('Management IP Address', 'managementIpAddress')}
                                    {infoRow('Platform ID', 'platformId')}
                                    {infoRow('Product ID', 'productId')}
                                    {infoRow('Serial Number', 'serialNumber')}
                                    {infoRow('Type', 'type')}
                                    {infoRow('UpTime', 'upTime')}
                                    
                                    <TableRow>
                                        <TableCell colSpan={2}>
                                            <hr />
                                        </TableCell>
                                    </TableRow>
                                    
                                    {infoRow('Collection Status', 'collectionStatus')}
                                    {infoRowArray('Connected Interface Name', 'connectedInterfaceName')}
                                    {infoRowArray('Connected Network Device Name', 'connectedNetworkDeviceName')}
                                    {infoRowArray('Connected Network Device Ip Address', 'connectedNetworkDeviceIpAddress')}
                                    {infoRow('Error Description', 'errorDescription')}
                                    {infoRow('Inventory Status Detail', 'inventoryStatusDetail')}
                                    {infoRow('Last Update Time', 'lastUpdateTime',`${props.device['lastUpdateTime']} seconds`)}
                                    {infoRow('Last Updated', 'lastUpdated')}
                                    {infoRow('Reachability Failure Reason', 'reachabilityFailureReason')}
                                    {infoRow('Reachability Status', 'reachabilityStatus')}
                                </TableBody>
                            </Table>
                        </Grid>
                        <Grid item md={4}>
                            <h4 style={{textAlign:'left'}}>Device Configuration</h4>
                            <Select
                                id='inventory-add-cli-select'
                                classNamePrefix='inventory-add-cli-select'
                                options={getCliOptions()} 
                                defaultValue={getDefaultCliOptions()}
                                closeMenuOnSelect={true}
                                menuPlacement={'bottom'}
                                styles={{
                                    container: (base) => ({
                                        ...base, 
                                        marginTop:10,
                                        marginBottom:25
                                    })
                                }}
                                placeholder='CLI Credential'
                                onChange={(option)=>{
                                    let settings = { ...device };
                                    settings['globalCredentialId'] = option.value;
                                    setState(settings);
                                }}
                            />
                        </Grid>
                    </Grid>
                </DialogContent>
                <DialogActions>
                    <Button
                        id='inventory-edit-delete'
                        onClick={()=>{
                            handleDelete(props.device.managementIpAddress);
                        }}
                        variant='outlined'
                        color='secondary'
                        style={{
                            marginRight: 'auto'
                        }}
                    >
                        Delete
                    </Button>
                    <Button id='inventory-edit-apply' onClick={handleSave} color='primary'>
                        Update
                    </Button>                    
                    <Button id='inventory-edit-cancel' onClick={handleClose} color='default'>
                        Cancel
                    </Button>

                </DialogActions>
            </Dialog>
        </>
    );
}

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//