//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableHead from '@material-ui/core/TableHead';
import TableRow from '@material-ui/core/TableRow';
import { faCog } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import _ from 'lodash';
import { makeStyles } from '@material-ui/core/styles';
import Edit from '../Edit';

//---------------------------------------------------------------------------//

library.add(faCog);

const useStyles = makeStyles({
    table: {
        minWidth: 650,
    },
    tableCell: {
        textAlign: 'left',
        whiteSpace: 'nowrap'
    }
});

const Items = (props) => {
    const rows = [];
    const classes = useStyles();

    _.forEach(props.state, (item, key) => {
        rows.push(
            <TableRow 
                key={key}
                hover={true}
            >
                <TableCell align='left' size='small'>
                    <Edit device={item} {...props} />
                </TableCell>
                <TableCell className={classes.tableCell} size='small' id={`inventory-tablecell-hostname-${key}`}>{item.hostname}</TableCell>
                <TableCell className={classes.tableCell} size='small' id={`inventory-tablecell-type-${key}`}>{item.type}</TableCell>
                <TableCell className={classes.tableCell} size='small' id={`inventory-tablecell-managementIpAddress-${key}`}>{item.managementIpAddress}</TableCell>
                <TableCell className={classes.tableCell} size='small' id={`inventory-tablecell-upTime-${key}`}>{item.upTime}</TableCell>
                <TableCell className={classes.tableCell} size='small' id={`inventory-tablecell-lastUpdated-${key}`}>{item.lastUpdated}</TableCell>
                <TableCell className={classes.tableCell} size='small' id={`inventory-tablecell-collectionStatus-${key}`}>{item.collectionStatus}</TableCell>
            </TableRow>
        )
    });
    return rows
};

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    const classes = useStyles();

    return (
        <Table className={classes.table}>
            <TableHead>
                <TableRow>
                    <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}></TableCell>
                    <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}>Hostname</TableCell>
                    <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}>Type</TableCell>
                    <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}>IP</TableCell>
                    <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}>Up Time</TableCell>
                    <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}>Last Updated</TableCell>
                    <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}>Collection Status</TableCell>
                </TableRow>
            </TableHead>
            <TableBody>
                { <Items {...props}/> }
            </TableBody>
        </Table>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//

