//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React, { useEffect } from 'react';
import Grid from '@material-ui/core/Grid';
import Container from '@material-ui/core/Container';
import Card from '@material-ui/core/Card';
import CardHeader from '@material-ui/core/CardHeader';
import CardContent from '@material-ui/core/CardContent';
import AppBar from '@material-ui/core/AppBar';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faBoxOpen, faHdd, faServer } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { useHistory } from 'react-router-dom';
import { connect } from 'react-redux';
import Table from './components/Table';

import Add from './components/Add';
import CommonApi from '../../Api';

//---------------------------------------------------------------------------//

library.add(faBoxOpen, faHdd, faServer);

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    let history = useHistory();
    const navigate = (view) => {
        history.push('/' + view);
    };

    useEffect(() => {
        props.dispatchUtil.fetchCliCredentials();
        // eslint-disable-next-line
    },[]); 
    

    useEffect(() => {
        props.dispatchUtil.fetchInventory();
        let timer = setInterval(()=>{
            props.dispatchUtil.fetchInventory();
        }, 10000);
        
        return () => {
            // Clean up the timer.
            clearTimeout(timer);
        };
    // eslint-disable-next-line
    },[]); 

    return (
        <>
            <AppBar
                position='static'
                elevation={0}   
            >
                <Container maxWidth='xl'>
                    <Tabs value={0} >
                        <Tab id='navbar-tab-inventory' onClick={navigate.bind(this, 'inventory')} label='Network Device' />
                        <Tab id='navbar-tab-discovery' onClick={navigate.bind(this, 'discovery')} label='Discovery' />
                        <Tab id='navbar-tab-credential' onClick={navigate.bind(this, 'credential')} label='Credentials' />
                    </Tabs>
                </Container>
            </AppBar>
            <Container maxWidth='lg'>
                <Grid 
                    container
                    direction='row'
                    justify='center'
                    alignItems='flex-start'
                >
                    <Card>
                        <CardHeader
                            avatar={
                                <FontAwesomeIcon icon='server' size='lg' fixedWidth />
                            }
                            title='Network Device'
                            action={
                                <Add {...props}/>
                            }
                            style={{
                                borderBottom: '1px #dde0e6 solid',
                                color:'#495057'
                            }}
                        />
                        <CardContent
                            style={{
                                overflow:'auto'
                            }}
                        >
                            <Table {...props}/>
                        </CardContent>
                    </Card>
                </Grid>
            </Container>
        </>
    );
};

//---------------------------------------------------------------------------//
// Redux Actions
//---------------------------------------------------------------------------//

const mapStateToProps = state => ({
    state: state.inventory,
    clis: state.clis
});

const mapDispatchToProps = dispatch => ({
    dispatchUtil: {
        fetchInventory: () => {
            CommonApi.Inventory.getNetworkDevices(dispatch);
        },

        addDevice: (device) => {
            CommonApi.Inventory.addNetworkDevice(dispatch, device);
        },

        removeDevice: ip => {
            CommonApi.Inventory.deleteNetworkDeviceByIp(dispatch, ip);
        },

        editDevice: (device) => {
            CommonApi.Inventory.updateNetworkDevice(dispatch, device);
        },

        changeDeviceDetail: (device) => {
            dispatch({
                type: 'INVENTORY_CHANGE',
                payload: {
                    device: {...device}
                }
            });
        },

        fetchCliCredentials: () => {
            CommonApi.Cli.getCliCredential(dispatch);
        }
    }
});

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default connect(
    mapStateToProps,
    mapDispatchToProps
)(Component);

//---------------------------------------------------------------------------//