//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import clsx from 'clsx';
import { makeStyles } from '@material-ui/core/styles';
import Grid from '@material-ui/core/Grid';
import Button from '@material-ui/core/Button';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faUnlockAlt, faEye, faEyeSlash } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { useHistory } from 'react-router-dom';
import Card from '@material-ui/core/Card';
import CardHeader from '@material-ui/core/CardHeader';
import CardContent from '@material-ui/core/CardContent';
import FormControl from '@material-ui/core/FormControl';
import IconButton from '@material-ui/core/IconButton';
import Input from '@material-ui/core/Input';
import InputLabel from '@material-ui/core/InputLabel';
import InputAdornment from '@material-ui/core/InputAdornment';
import Visibility from '@material-ui/icons/Visibility';
import VisibilityOff from '@material-ui/icons/VisibilityOff';

//---------------------------------------------------------------------------//

library.add(faUnlockAlt, faEye, faEyeSlash);

const useStyles = makeStyles(theme => ({
    margin: {
        margin: theme.spacing(1),
    },
    withoutLabel: {
        marginTop: theme.spacing(3),
    },
    textField: {
        width: 200,
    }
}));

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    const history = useHistory();
    const classes = useStyles();
    const [state, setState] = React.useState({
        username: '',
        password: '',
        showPassword: false
    });

    const handleChange = (event, key) => {
        let credential = { ...state };
        let value = event.target.value;
        credential[key] = value;
        setState(credential);
    };

    const handleLogin = (event) => {
        props.dispatchUtil.login({
            username: state.username,
            password: state.password,
        }, history);

    };

    const handleClickShowPassword = () => {
        setState({ ...state, showPassword: !state.showPassword });
    };
    
    const handleMouseDownPassword = event => {
        event.preventDefault();
    };

    const keyPressed = (event) => {
        if (event.key === "Enter") {
            handleLogin();
        }
    };

    return (

        <Card
            style={{
                padding: 10
            }}
        >
            <CardHeader
                avatar={
                    <img
                    style={{
                        width: 50,
                        height: 50,
                        marginRight: 10
                    }}
                    src={'images/logo.svg'}
                    alt='Cisco Logo'
                    />
                }
                title='Network Controller'
                subheader='User Login'
                style={{
                    marginBottom: 20
                }}
            />
            <CardContent>
                <Grid container spacing={2}>

                    <FormControl className={clsx(classes.margin, classes.textField)}
                        onKeyPress={keyPressed}
                    >
                        <InputLabel htmlFor="login-username">Username</InputLabel>
                        <Input
                            id="login-username"
                            type='text'
                            value={state.username}
                            onChange={event => handleChange(event, 'username')}
                        />
                    </FormControl>
                    <FormControl className={clsx(classes.margin, classes.textField)}
                        onKeyPress={keyPressed}
                    >
                        <InputLabel htmlFor="login-password">Password</InputLabel>
                        <Input
                            id="login-password"
                            type={state.showPassword ? 'text' : 'password'}
                            value={state.password}
                            onChange={event => handleChange(event, 'password')}
                            endAdornment={
                                <InputAdornment position="end">
                                    <IconButton
                                        aria-label="toggle password visibility"
                                        onClick={handleClickShowPassword}
                                        onMouseDown={handleMouseDownPassword}
                                    >
                                    {state.showPassword ? <Visibility /> : <VisibilityOff />}
                                    </IconButton>
                                </InputAdornment>
                            }
                        />
                    </FormControl>
                </Grid>
                <Button                     
                    style={{
                        float:'right',
                        marginTop: 20,
                        marginBottom: 10
                    }}
                    variant='contained' 
                        id='login-submit'
                        onClick={handleLogin} 
                        color='primary'>
                        <FontAwesomeIcon icon='unlock-alt' size='1x' fixedWidth />
                        <span style={{marginLeft:5}}>
                            LOG IN
                        </span>
                </Button>
            </CardContent>
        </Card>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//