//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import clsx from 'clsx';
import { makeStyles } from '@material-ui/core/styles';
import Button from '@material-ui/core/Button';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faUserPlus, faEye, faEyeSlash } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { useHistory } from 'react-router-dom';
import Card from '@material-ui/core/Card';
import CardHeader from '@material-ui/core/CardHeader';
import CardContent from '@material-ui/core/CardContent';
import FormControl from '@material-ui/core/FormControl';
import IconButton from '@material-ui/core/IconButton';
import Input from '@material-ui/core/Input';
import InputLabel from '@material-ui/core/InputLabel';
import InputAdornment from '@material-ui/core/InputAdornment';
import Visibility from '@material-ui/icons/Visibility';
import VisibilityOff from '@material-ui/icons/VisibilityOff';
import Toast from '../../../components/Toast';

//---------------------------------------------------------------------------//

library.add(faUserPlus, faEye, faEyeSlash);

const useStyles = makeStyles(theme => ({
    margin: {
        margin: theme.spacing(1),
    },
    withoutLabel: {
        marginTop: theme.spacing(3),
    }
}));

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    const history = useHistory();
    const classes = useStyles();
    const [state, setState] = React.useState({
        username: '',
        password: '',
        confirmPassword: '',
        showPassword: false,
        showConfirmPassword: false
    });

    const handleChange = (event, key) => {
        let credential = { ...state };
        let value = event.target.value;
        credential[key] = value;
        setState(credential);
    };
    
    const handleSetup = (event) => {

        if (state.username===''){
            Toast.error('Username cannot be empty');
        }
        else if(state.password===''){
            Toast.error('Password cannot be empty');
        }
        else if (state.password!==state.confirmPassword){
            Toast.error('Confirm password does not match');
        }
        else {
            props.dispatchUtil.setup({
                username: state.username,
                password: state.password
            }, history);
        }
    };

    const handleClickShowPassword = () => {
        setState({ ...state, showPassword: !state.showPassword });
    };
    
    const handleClickShowConfirmPassword = () => {
        setState({ ...state, showConfirmPassword: !state.showConfirmPassword });
    };
    
    const handleMouseDownPassword = event => {
        event.preventDefault();
    };

    const keyPressed = (event) => {
        if (event.key === "Enter") {
            handleSetup();
        }
    };

    return (
        <>
            <Card
                style={{
                    padding: 10,
                    minWidth: 350
                }}
                >
                <CardHeader
                    avatar={
                        <img
                            style={{
                                width: 50,
                                height: 50,
                                marginRight: 10
                            }}
                            src={'images/logo.svg'}
                            alt='Cisco Logo'
                        />
                    }
                    title='Network Controller'
                    subheader='User Setup'
                    style={{
                        marginBottom: 20
                    }}
                />
                <CardContent>
                    <span
                        style={{
                            color: '#495057',
                            position: 'relative',
                            bottom: 25,
                            left: 10
                        }}
                    >
                        Register a new user account
                    </span>

                    <div>
                        <FormControl className={clsx(classes.margin)}
                            onKeyPress={keyPressed}
                            style={{
                                width: 300
                            }}
                        >
                            <InputLabel htmlFor="setup-username">Username</InputLabel>
                            <Input
                                id="setup-username"
                                type='text'
                                value={state.username}
                                onChange={event => handleChange(event, 'username')}
                            />
                        </FormControl>
                    </div>
                    <div>
                        <FormControl className={clsx(classes.margin)}
                            onKeyPress={keyPressed}
                            style={{
                                width: 300
                            }}
                        >
                            <InputLabel htmlFor="setup-password">Password</InputLabel>
                            <Input
                                id="setup-password"
                                type={state.showPassword ? 'text' : 'password'}
                                value={state.password}
                                onChange={event => handleChange(event, 'password')}
                                endAdornment={
                                    <InputAdornment position="end">
                                        <IconButton
                                            aria-label="toggle password visibility"
                                            onClick={handleClickShowPassword}
                                            onMouseDown={handleMouseDownPassword}
                                        >
                                        {state.showPassword ? <Visibility /> : <VisibilityOff />}
                                        </IconButton>
                                    </InputAdornment>
                                }
                            />
                        </FormControl>
                    </div>
                    <div>
                        <FormControl className={clsx(classes.margin)}
                            onKeyPress={keyPressed}
                            style={{
                                width: 300
                            }}
                        >
                            <InputLabel htmlFor="setup-confirmPassword">Confirm Password</InputLabel>
                            <Input
                                id="setup-confirmPassword"
                                type={state.showConfirmPassword ? 'text' : 'password'}
                                value={state.confirmPassword}
                                onChange={event => handleChange(event, 'confirmPassword')}
                                endAdornment={
                                    <InputAdornment position="end">
                                        <IconButton
                                            aria-label="toggle confirm password visibility"
                                            onClick={handleClickShowConfirmPassword}
                                            onMouseDown={handleMouseDownPassword}
                                        >
                                        {state.showConfirmPassword ? <Visibility /> : <VisibilityOff />}
                                        </IconButton>
                                    </InputAdornment>
                                }
                            />
                        </FormControl>
                    </div>
                    <Button                     
                        style={{
                            float:'right',
                            marginTop: 10,
                            marginRight: 5,
                            marginBottom: 10
                        }}
                        variant='contained' 
                            id='setup-submit'
                            onClick={handleSetup} 
                            color='primary'>
                            <FontAwesomeIcon icon='user-plus' size='1x' fixedWidth />
                            <span style={{marginLeft:5}}>
                                Setup
                            </span>
                    </Button>
                </CardContent>
            </Card>
        </>
    );
}

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//