//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import { makeStyles } from '@material-ui/core/styles';
import {
    Drawer, 
    List,
    ListItem,
    ListItemIcon,
    ListItemText,
    IconButton
} from '@material-ui/core';
import { Menu as MenuIcon } from '@material-ui/icons';
import { useHistory } from 'react-router-dom';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faTachometerAlt, faServer, faGavel, faBoxOpen, faHeartbeat, faNetworkWired, faSatelliteDish, faRunning, faBook } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import _ from 'lodash';

//---------------------------------------------------------------------------//

library.add(faTachometerAlt, faServer, faGavel, faBoxOpen, faHeartbeat, faNetworkWired, faSatelliteDish, faRunning, faBook);

const useStyles = makeStyles({
    list: {
        width: 230,
        position: 'relative',
    }
});

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    
    const classes = useStyles();
    const [state, setState] = React.useState({
        left: false
    });

    const toggleDrawer = (open) => event => {
        setState({ ...state,
            left: open 
        });
    };
    
    let history = useHistory();

    const navigate = (view, side) => {
        history.push('/' + view);
    };

    const highlight = (views) => {

        let isView = false;

        _.forEach(views, (view)=>{
            if(`/${view}` === history.location.pathname){
                isView = true;
            }
        });

        if(isView){
            return {
                background:'#323c48',
                color:'#60b6ef'
            }
        }else{
            return {
                color:'white'
            }
        }
    };

    const sideList = () => (
        <div
            className={classes.list}
            role='presentation'
            style={{
                height: '100%',
                background: '#45515f'
            }}
        >
            <List
                dense={true}
                onClick={toggleDrawer(false)}
            >

                <ListItem
                    id='drawer-listItem-dashboard'
                    button 
                    key={'Dashboard'}
                    onClick={navigate.bind(this, 'dashboard')}
                    style={highlight(['dashboard'])}
                >
                    <ListItemIcon to='/'>
                        <FontAwesomeIcon style={highlight(['dashboard'])} icon='tachometer-alt' size='lg' fixedWidth />
                    </ListItemIcon>
                    <ListItemText primary={'Dashboard'} />
                </ListItem>

                <ListItem 
                    id='drawer-listItem-provisioning'
                    button
                    key={'Provisioning'}
                    onClick={navigate.bind(this, 'inventory')}
                    style={highlight(['inventory', 'discovery', 'credential'])}
                >
                    <ListItemIcon to='/'>
                        <FontAwesomeIcon style={highlight(['inventory', 'discovery', 'credential'])} icon='server' size='lg' fixedWidth />
                    </ListItemIcon>
                    <ListItemText primary={'Provisioning'} />
                </ListItem>

                <ListItem 
                    id='drawer-listItem-assurance'
                    button 
                    key={'Assurance'}
                    onClick={navigate.bind(this, 'assurance')}
                    style={highlight(['assurance', 'host', 'topology', 'pathtrace'])}
                >
                    <ListItemIcon to='/'>
                        <FontAwesomeIcon style={highlight(['assurance', 'host', 'topology', 'pathtrace'])} icon='heartbeat' size='lg' fixedWidth />
                    </ListItemIcon>
                    <ListItemText primary={'Assurance'} />
                </ListItem>

                <ListItem
                    id='drawer-listItem-qos'
                    button 
                    key={'Qos'}
                    onClick={navigate.bind(this, 'qos')}
                    style={highlight(['qos'])}
                >
                    <ListItemIcon to='/'>
                        <FontAwesomeIcon style={highlight(['policy'])} icon='gavel' size='lg' fixedWidth />
                    </ListItemIcon>
                    <ListItemText primary={'Policy'} />
                </ListItem>

                <ListItem
                    id='drawer-listItem-swagger'
                    button 
                    key={'swagger'}
                    onClick={navigate.bind(this, 'swagger')}
                    style={highlight(['swagger'])}
                >
                    <ListItemIcon to='/'>
                        <FontAwesomeIcon style={highlight(['swagger'])} icon='book' size='lg' fixedWidth />
                    </ListItemIcon>
                    <ListItemText primary={'API Docs'} />
                </ListItem>

            </List>
        </div>
    );

    return (
        <div>
            <IconButton
                id='drawer-toggle-button'
                onClick={toggleDrawer(true)}
                edge='start' 
                className={classes.menuButton} 
                color='inherit' 
                aria-label='menu'
            >
                <MenuIcon />
            </IconButton>

            <Drawer 
                open= { state.left } 
                onClose={ toggleDrawer(false)}
            >
                {
                    sideList()
                }
            </Drawer>
        </div>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//
