//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faUser, faCaretDown, faSignOutAlt, faUserCog, faCogs } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { useHistory } from 'react-router-dom';
import MenuItem from '@material-ui/core/MenuItem';
import Menu from '@material-ui/core/Menu';

//---------------------------------------------------------------------------//

library.add(faUser, faCaretDown, faSignOutAlt, faUserCog, faCogs);

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    let history = useHistory();
    
    const [state, setState] = React.useState({
        anchorEl: null,
        open: false,
    });

    const handleClick = event => {
        setState({ 
            ...state, 
            open: true,
            anchorEl: event.currentTarget
        });
    };

    const handleRequestClose = () => {
        setState({
            ...state, 
            open: false
        });
    };

    const navigate = (view) => {
        history.push('/' + view);
    };
    
    const highlight = (view) => {
        if(`/${view}` === history.location.pathname){
            return {
                color:'#1e4471',
            }
        }else{
            return {
            }
        }
    };

    const userButtonHighlight = () => {
        let result = {
            background:'none',
            border:'none',
            outline: 0,
            height: 30
        }

        let pathname = history.location.pathname;

        if(pathname==='/user'){
            result = {...result, color: '#1e4471'}
        }
        return result;
    };

    return (
        <div>
            <button 
                aria-owns={state.open ? 'simple-menu' : null}
                aria-haspopup='true'
                onClick={handleClick}
                style={userButtonHighlight()}
            >
                <FontAwesomeIcon icon='user' size='1x' />
                <span
                    id='navbar-user-username'
                    style={{
                        marginLeft: 8,
                        marginRight: 8,
                        position: 'relative',
                        fontSize: 18,
                        color: '#1e4471',
                        top: 0
                    }}
                >
                    {props.state.username}
                </span>
                <FontAwesomeIcon icon='caret-down' size='xs'/>
            </button>

            <Menu
                id='navbar-user-menu'
                anchorEl={state.anchorEl}
                open={state.open}
                onClose={handleRequestClose}
            >
                <MenuItem
                    id='navbar-user-menuitem-user'
                    onClick={()=>{
                        handleRequestClose();
                        navigate('user');
                    }}
                    style={highlight('user')}
                >
                    <FontAwesomeIcon icon='user-cog' size='1x' fixedWidth style={{marginRight:10}}/>
                    <span>
                        User Management
                    </span>
                </MenuItem>
                <MenuItem
                    id='navbar-user-menuitem-logout'
                    onClick={()=>{
                        props.dispatchUtil.logout();
                        handleRequestClose();
                        navigate('login');
                    }}
                >
                    <FontAwesomeIcon icon='sign-out-alt' size='1x' fixedWidth style={{marginRight:10}}/>
                    <span>
                        Logout
                    </span>
                </MenuItem>
            </Menu>
        </div>
    );
}

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component

//---------------------------------------------------------------------------//