//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import { makeStyles } from '@material-ui/core/styles';
import AppBar from '@material-ui/core/AppBar';
import Toolbar from '@material-ui/core/Toolbar';
import IconButton from '@material-ui/core/IconButton';
import Typography from '@material-ui/core/Typography';
import MenuItem from '@material-ui/core/MenuItem';
import Menu from '@material-ui/core/Menu';
import MoreIcon from '@material-ui/icons/MoreVert';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faCogs, faUser, faSignOutAlt } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { connect } from 'react-redux';
import { useHistory } from 'react-router-dom';
import Drawer from './components/Drawer';
import User from './components/User';
import CommonApi from '../../Api';

//---------------------------------------------------------------------------//

library.add(faCogs, faUser, faSignOutAlt);

const useStyles = makeStyles(theme => ({
    grow: {
        flexGrow: 1,
    },
    appbar:{
        background: 'white',
        color: '#1e4471',
        marginBottom: 0
    },
    menuButton: {
        marginRight: theme.spacing(2),
    },
    title: {
        display: 'none',
        [theme.breakpoints.up('sm')]: {
            display: 'block',
        },
    },
    inputRoot: {
        color: 'inherit',
    },
    inputInput: {
        padding: theme.spacing(1, 1, 1, 7),
        transition: theme.transitions.create('width'),
        width: '100%',
        [theme.breakpoints.up('md')]: {
            width: 200,
        },
    },
    sectionDesktop: {
        display: 'none',
        [theme.breakpoints.up('md')]: {
            display: 'flex',
        },
    },
    sectionMobile: {
        display: 'flex',
        [theme.breakpoints.up('md')]: {
            display: 'none',
        },
    },
}));

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const classes = useStyles();
    const [mobileMoreAnchorEl, setMobileMoreAnchorEl] = React.useState(null);

    const isMobileMenuOpen = Boolean(mobileMoreAnchorEl);

    let history = useHistory();

    const navigate = (view) => {
        history.push('/' + view);
    }

    const handleMobileMenuClose = () => {
        setMobileMoreAnchorEl(null);
    };

    const handleLogoClick = () => {
        navigate('dashboard');
    };

    const handleMobileMenuOpen = event => {
        setMobileMoreAnchorEl(event.currentTarget);
    };

    const renderMobileMenu = (
        <Menu
            id='navbar-mobile-menu'
            anchorEl={mobileMoreAnchorEl}
            anchorOrigin={{ vertical: 'top', horizontal: 'right' }}
            keepMounted
            transformOrigin={{ vertical: 'top', horizontal: 'right' }}
            open={isMobileMenuOpen}
            onClose={handleMobileMenuClose}
        >
            <MenuItem
                id='navbar-mobile-menuitem-user'
                dense={true}
                onClick={()=>{
                    handleMobileMenuClose();
                    navigate('user');
                }}
            >
                <IconButton
                    aria-haspopup='true'
                    color='inherit'
                >
                    <FontAwesomeIcon icon='user-cog' size='1x' fixedWidth />
                </IconButton>
                <p>User Management</p>
            </MenuItem>
            <MenuItem
                id='navbar-mobile-menuitem-logout'
                dense={true}
                onClick={()=>{
                    props.dispatchUtil.logout();
                    handleMobileMenuClose();
                    navigate('login');
                }}
            >
                <IconButton
                    aria-haspopup='true'
                    color='inherit'
                >
                    <FontAwesomeIcon icon='sign-out-alt' size='1x' fixedWidth />
                </IconButton>
                <p>Log Out</p>
            </MenuItem>
        </Menu>
    );

    return (
        <div className={classes.grow}>
            <AppBar 
                position='static'
                elevation={1}
                className={classes.appbar}
            >
                <Toolbar>
                    <Drawer />
                        <input
                            type='image'
                            onClick={handleLogoClick}
                            style={{
                                width: 50,
                                height: 50,
                                marginRight: 10,
                                cursor: 'pointer'
                            }}
                            src={'images/logo.svg'}
                            alt='Cisco Logo'
                            id='navbar-logo'
                        />
                        <Typography 
                            onClick={handleLogoClick}
                            className={classes.title} 
                            variant='h6' 
                            noWrap
                            style={{
                                cursor: 'pointer'
                            }}
                        >
                            Network Controller
                        </Typography>
                    <div className={classes.grow} />
                    <div className={classes.sectionDesktop}>
                        <User {...props} /> 
                    </div>
                    <div className={classes.sectionMobile}>
                        <IconButton
                            aria-label='show more'
                            aria-controls='navbar-mobile-menu'
                            aria-haspopup='true'
                            onClick={handleMobileMenuOpen}
                            color='inherit'
                        >
                            <MoreIcon />
                        </IconButton>
                    </div>
                </Toolbar>
            </AppBar>
            {renderMobileMenu}
        </div>
    );
}

//---------------------------------------------------------------------------//
// Redux Actions
//---------------------------------------------------------------------------//

const mapStateToProps = state => ({
    state: state.app
});

const mapDispatchToProps = (dispatch) => {
    return {
        dispatchUtil: {
            logout: () => {
                CommonApi.Login.deleteTicket(dispatch);
            }
        }
    }
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default connect(
    mapStateToProps,
    mapDispatchToProps
)(Component);

//---------------------------------------------------------------------------//