//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Button from '@material-ui/core/Button';
import { faPlus } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { makeStyles } from '@material-ui/core/styles';
import TextField from '@material-ui/core/TextField';
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogTitle from '@material-ui/core/DialogTitle';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'

//---------------------------------------------------------------------------//

library.add(faPlus);

const useStyles = makeStyles(theme => ({
    button: {
        position: 'relative',
        top: 10,
        right: 5
    },
}));

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const classes = useStyles();
    const [open, setOpen] = React.useState(false);

    const [pathtrace, setState] = React.useState({
        'destIP': '',
        'sourceIP': ''
    });

    const handleClickOpen = () => {
        setOpen(true);
    };

    const handleClose = () => {
        setOpen(false);
    };

    const handleChange = (event, key) => {
        let config = { ...pathtrace };
        let value = event.target.value;
        config[key] = value;
        setState(config);
    };

    const handleSubmit = () => {
        props.dispatchUtil.add(pathtrace);
        setOpen(false);
    };

    return (
        <div>
            <Button    
                id='pathtrace-add-button'
                className={classes.button} 
                variant='contained' 
                color='primary' 
                size='small'          
                onClick={handleClickOpen}       
                style={{
                    position: 'relative',
                    top: 5
                }}
            >
                <FontAwesomeIcon icon='plus' size='1x' fixedWidth />
                <span style={{marginLeft:5}}>
                    Path
                </span>
            </Button>
            <Dialog open={open} onClose={handleClose} aria-labelledby='form-dialog-title'>
                <DialogTitle id='form-dialog-title'>New Path</DialogTitle>
                <DialogContent>
                    <TextField
                        id='pathtrace-add-sourceIp'
                        margin='dense'
                        label='Source'
                        type='text'
                        fullWidth
                        onChange={event => handleChange(event, 'sourceIP')}
                    />
                    <TextField
                        id='pathtrace-add-destIp'
                        margin='dense'
                        label='Destination'
                        type='text'
                        fullWidth
                        onChange={event => handleChange(event, 'destIP')}
                    />
                </DialogContent>
                <DialogActions>
                <Button id='pathtrace-add-cancel' onClick={handleClose} color='default'>
                    Cancel
                </Button>
                <Button id='pathtrace-add-submit' onClick={handleSubmit} color='primary'>
                    Okay
                </Button>
                </DialogActions>
            </Dialog>
        </div>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//