//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import List from '@material-ui/core/List';
import ListItem from '@material-ui/core/ListItem';
import Item from './Item';
import _ from 'lodash';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faCheck, faTimes, faSync } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';

import util from '../../../../utils';

//---------------------------------------------------------------------------//

library.add(faCheck, faTimes, faSync);

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const conditionIcon = (condition) => {
        if(condition === 'COMPLETED'){
            return <FontAwesomeIcon icon='check' size='lg' fixedWidth style={{color:'#52ae67'}}/>;
        }
        
        else if(condition === 'INPROGRESS'){
            return <FontAwesomeIcon icon='sync' size='lg' spin fixedWidth style={{color:'#ffb53e'}}/>;
        }

        else{
            return <FontAwesomeIcon icon='times' size='lg' fixedWidth style={{color:'#ff6384'}}/>;
        }
    };

    return (
        <>
            <span
                id='pathtrace-detail-status'
            >
                <b>{'Condition: '}</b>
                <span>
                    {props.state.detail.request.status}
                </span>
            </span>
            <div
                style={{
                    marginLeft: 10,
                    display: 'inline',
                }}
            >
                {conditionIcon(props.state.detail.request.status)}
            </div>
            <span
                id='pathtrace-detail-lastUpdate'
                style={{
                    lineHeight:2,
                    display: 'block',
                }}
            >
                <b>{'Last Updated: '}</b>
                <span>
                    {props.state.detail.lastUpdate}
                </span>
            </span>
            <hr 
                style={{
                    marginTop: 20
                }}
            />
            <List>
                {_.map(props.state.detail.networkElementsInfo, (row, index) => {

                    let image = _.find(util.images, (img)=>{
                        return img.type === row.type;
                    });

                    if(image===undefined){
                        image = util.images[0];
                    }

                    return(
                        <ListItem 
                            button={false}
                            selected={false}
                            key={index}
                        >
                            <Item 
                                index={index}
                                image={image.name}
                                name={row.name}
                                ip={row.ip}
                                type={row.type}
                                ingress={row.ingressInterface}
                                egress={row.egressInterface}
                                last={props.state.detail.networkElementsInfo.length-1===index}
                            />
                        </ListItem>
                    )
                })}
            </List>
        </>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//