//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React, { useEffect } from 'react';
import Grid from '@material-ui/core/Grid';
import Container from '@material-ui/core/Container';
import Card from '@material-ui/core/Card';
import CardHeader from '@material-ui/core/CardHeader';
import CardContent from '@material-ui/core/CardContent';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faRunning, faMapSigns, faTrash } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import AppBar from '@material-ui/core/AppBar';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import { useHistory } from 'react-router-dom';
import { connect } from 'react-redux';
import Button from '@material-ui/core/Button';
import _ from 'lodash';

import List from './components/List';
import Detail from './components/Detail';
import Add from './components/Add';

import CommonApi from '../../Api';

//---------------------------------------------------------------------------//

library.add(faRunning, faMapSigns, faTrash);

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    let history = useHistory();
    const navigate = (view) => {
        history.push('/' + view);
    };

    // Do once onload fetch list.

    useEffect(() => {
        props.dispatchUtil.fetchList();
    // eslint-disable-next-line
    },[]); 

    // When detail changes and in progress, do a fetch.

    useEffect(() => {
        
        let timer = setInterval(()=>{ 
            if(!_.isEmpty(props.state.detail) && props.state.detail.request.status==='INPROGRESS'){
                let id = props.state.detail.request.id;
                props.dispatchUtil.getPathTrace(id);
            } 
        }, 5000);
        
        return () => {
            // Clean up the timer.
            clearTimeout(timer);
        };
        
    // eslint-disable-next-line
    },[props.state.detail]); 

    const remove = () => {
        props.dispatchUtil.remove(props.state.detail.request.id);
    };

    const getDetail = () => {

        if(!_.isEmpty(props.state.detail)){
            return(
                <Grid item md={8}
                    style={{
                        width:'100%'
                    }}
                >
                    <Card>
                        <CardHeader
                            avatar={
                                <FontAwesomeIcon icon='map-signs' size='lg' fixedWidth />
                            }
                            title='Route'
                            style={{
                                borderBottom: '1px #dde0e6 solid',
                                color: '#495057'
                            }}
                            action={
                                <Button 
                                    id='pathtrace-delete-button'
                                    color='default'
                                    variant='contained' 
                                    size='small'
                                    onClick={remove}
                                    style={{
                                        position: 'relative',
                                        top: 5,
                                        marginRight: '5px'
                                    }}
                                >
                                    <FontAwesomeIcon icon='trash' size='1x' fixedWidth />
                                    <span style={{marginLeft:5}}>
                                        Delete
                                    </span>
                                </Button>
                            }
                        />
                        <CardContent>
                            <Detail {...props} />
                        </CardContent>
                    </Card>
                </Grid>
            );
        }else{
            return null;
        }
    };

    return (
        <>
            <AppBar
                position='static'
                elevation={0}
            >
                <Container maxWidth='xl'>
                    <Tabs value={3} >
                        <Tab id='navbar-tab-assurance' onClick={navigate.bind(this, 'assurance')} label='Assurance' />
                        <Tab id='navbar-tab-hosts' onClick={navigate.bind(this, 'host')} label='Hosts' />
                        <Tab id='navbar-tab-topology' onClick={navigate.bind(this, 'topology')} label='Topology' />
                        <Tab id='navbar-tab-pathtrace' onClick={navigate.bind(this, 'pathtrace')}label='Path Trace' />
                    </Tabs>
                </Container>
            </AppBar>
            <Container 
                maxWidth='lg'
            >
                <Grid 
                    container
                    direction='row'
                    justify='center'
                    spacing={2}
                >
                    <Grid item md={4}
                        style={{
                            width:'100%'
                        }}
                    >
                        <Card>
                            <CardHeader
                                avatar={
                                    <FontAwesomeIcon icon='running' size='lg' fixedWidth />
                                }
                                title='Path Trace'
                                action={
                                    <Add {...props} />
                                }
                                style={{
                                    borderBottom: '1px #dde0e6 solid',
                                    color: '#495057'
                                }}
                            />
                            <CardContent>
                                <List {...props} />
                            </CardContent>
                        </Card>
                    </Grid>
                    {getDetail()}
                </Grid>
            </Container>
        </>

    );
};

//---------------------------------------------------------------------------//
// Redux Actions
//---------------------------------------------------------------------------//

const mapStateToProps = state => ({
    state: state.pathtrace
});

const mapDispatchToProps = dispatch => ({

    dispatchUtil: {

        fetchList: () => {
            CommonApi.Pathtrace.flowAnalysis(dispatch);
        },

        getPathTrace: (config) => {
            CommonApi.Pathtrace.getFullFlowAnalysisResult(dispatch, config);
        },

        add: (config) => {
            CommonApi.Pathtrace.initiateFlowAnalysis(dispatch, config);
        },

        remove: (id) => {
            CommonApi.Pathtrace.deleteFlowAnalysis(dispatch, id);
        }
    }
});

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default connect(
    mapStateToProps,
    mapDispatchToProps
)(Component);

//---------------------------------------------------------------------------//