//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Button from '@material-ui/core/Button';
import { faPlus } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { makeStyles } from '@material-ui/core/styles';
import TextField from '@material-ui/core/TextField';
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogTitle from '@material-ui/core/DialogTitle';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import Select from 'react-select';
import _ from 'lodash';

//---------------------------------------------------------------------------//

library.add(faPlus);

const useStyles = makeStyles(theme => ({
    root: {
        overflow: 'visible'
    },
    button: {
        position: 'relative',
        top: 10,
        right: 5
    },
}));

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    const classes = useStyles();
    const [open, setOpen] = React.useState(false);
    
    const [policy, setState] = React.useState({
        policyName: '',
        scopeName: '',
        protocols: '',
    });

    const handleClickOpen = () => {
        setOpen(true);
    };

    const handleClose = () => {
        setOpen(false);
    };

    const getProtocolOptions = () => {
        let result = [];
        _.forEach(props.state.qos.protocols, (protocol)=>{
            result.push({
                label: protocol.name,
                value:  protocol.id
            });
        });
        return result;
    };

    const getScopeOptions = () => {
        let result = [];
        _.forEach(props.state.qos.scopes, (scope)=>{
            result.push({
                label: scope.policyTag,
                value:  scope.policyTag
            });
        });
        return result;
    };

    const handleSubmit = () => {
        setOpen(false);
        props.dispatchUtil.addPolicy({
            actions:['SET_PROPERTY'],
            policyName: policy.policyName,
            policyScope: policy.scopeName,
            policyActionProperty: {
                relevanceLevel: policy.relevanceLevel
            },
            policyResource:{
                applications: policy.protocols
            }
        });
    };
    
    const handleChange = (event, key) => {
        let config = { ...policy };
        let value = event.target.value;
        config[key] = value;
        setState(config);
    };

    return (
        <div>
            <Button    
                id='qos-policy-add-button'
                className={classes.button} 
                variant='contained' 
                color='primary' 
                size='small'          
                onClick={handleClickOpen}       
                style={{
                    position: 'relative',
                    top: 5
                }}
            >
                <FontAwesomeIcon icon='plus' size='1x' fixedWidth />
                <span style={{marginLeft:5}}>
                    Policy
                </span>
            </Button>
            <Dialog 
                open={open} 
                onClose={handleClose} 
                fullWidth 
                maxWidth='xs'
                scroll= 'body'
                classes={{ 
                    paperScrollBody: classes.root 
                }}
            >
                <DialogTitle id='qos-scope-add-title'>New Policy</DialogTitle>
                <DialogContent
                    className={classes.root}
                >
                    <TextField
                        margin='dense'
                        id='qos-policy-add-name'
                        label='Name'
                        type='text'
                        value={policy.policyName}
                        onChange={event => handleChange(event, 'policyName')}
                        fullWidth
                    />

                    <div
                        style={{
                            position: 'relative',
                            width: '100%',
                            marginTop: 20,
                            marginBottom: 20
                        }}
                    >
                        <Select
                            name='relevance'
                            options={[
                                { label: 'Business-Relevant', value: 'Business-Relevant' },
                                { label: 'Business-Irrelevant', value: 'Business-Irrelevant' },
                                { label: 'Default', value: 'Default' }
                            ]}
                            placeholder="Select Relevance Level"
                            id='qos-policy-add-relevance-select'
                            classNamePrefix='qos-policy-add-relevance-select'
                            onChange={(option)=>{
                                setState({
                                    ...policy, 
                                    relevanceLevel: option.value
                                });
                            }}
                        />
                    </div>
                    <div
                        style={{
                            position: 'relative',
                            width: '100%',
                            marginTop: 20,
                            marginBottom: 20
                        }}
                    >
                        <Select
                            name='protocols'
                            isMulti
                            options={getProtocolOptions()}
                            placeholder="Select Protocols"
                            id='qos-policy-add-protocol-select'
                            classNamePrefix='qos-policy-add-protocol-select'
                            onChange={(options)=>{
                                let protocols = [];
                                _.forEach(options, (option)=>{
                                    protocols.push({
                                        id: option.value,
                                        name: option.name
                                    });
                                });
                                setState({
                                    ...policy, 
                                    protocols: protocols
                                });
                            }}
                        />
                    </div>
                    <div
                        style={{
                            position: 'relative',
                            width: '100%',
                            marginTop: 20,
                            marginBottom: 20
                        }}
                    >
                        <Select
                            name='scope'
                            options={getScopeOptions()}
                            placeholder="Select Scope"
                            id='qos-policy-add-scope-select'
                            classNamePrefix='qos-policy-add-scope-select'
                            onChange={(option)=>{
                                setState({
                                    ...policy, 
                                    scopeName: option.value
                                });
                            }}
                        />
                    </div>
                </DialogContent>
                <DialogActions>
                <Button id='qos-policy-add-cancel' onClick={handleClose} color='default'>
                    Cancel
                </Button>
                <Button id='qos-policy-add-submit' onClick={handleSubmit} color='primary'>
                    Add
                </Button>
                </DialogActions>
            </Dialog>
        </div>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//
