//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Button from '@material-ui/core/Button';
import { faCog } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogTitle from '@material-ui/core/DialogTitle';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import Select from 'react-select';
import { makeStyles } from '@material-ui/core/styles';
import _ from 'lodash';

//---------------------------------------------------------------------------//

library.add(faCog);

const useStyles = makeStyles(theme => ({
    root: {
        overflow: 'visible'
    }
}));

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    const classes = useStyles();

    const [open, setOpen] = React.useState(false);

    let initialConfig = {
        id: props.policy.id,
        policyName: props.policy.policyName,
        protocols: props.policy.policyResource.applications
    };
    
    const [policy, setState] = React.useState(initialConfig);

    const handleClickOpen = () => {
        setOpen(true);
        setState(initialConfig)
    };

    const handleClose = () => {
        setOpen(false);
    };

    const deletePolicy = () => {
        setOpen(false);
        props.dispatchUtil.deletePolicy(policy.id);
    };

    const handleSubmit = () => {
        props.dispatchUtil.updatePolicy({
            id: policy.id,
            payload: {
                applications: policy.protocols
            }
        });
        setOpen(false);
    };
    
    const getProtocolOptions = () => {
        let result = [];
        _.forEach(props.state.qos.protocols, (protocol)=>{
            result.push({
                label: protocol.name,
                value:  protocol.id
            });
        });
        return result;
    };

    const getDefaultProtocolOptions = (options) => {
        let result = [];
        _.forEach(policy.protocols, (protocol)=>{
            result.push({
                label: protocol.appName,
                value:  protocol.id
            });
        });
        return result;
    };

    return (
        <div>
            <Button    
                id={`qos-policy-edit-button-${props.index}`}
                variant='contained' 
                color='default' 
                size='small'
                fullWidth={true}  
                onClick={handleClickOpen}
                style={{
                    maxWidth: 25,
                    maxHeight: 25, 
                    minWidth: 25, 
                    minHeight: 25
                }}
            >
                <FontAwesomeIcon 
                    icon='cog' 
                    size='1x' 
                />
            </Button>

            <Dialog 
                open={open} 
                onClose={handleClose} 
                scroll= 'body'
                classes={{ 
                    paperScrollBody: classes.root 
                }}
                maxWidth='xs'
                fullWidth
            >
                <DialogTitle>Edit Policy</DialogTitle>
                <DialogContent
                    className={classes.root}
                    style={{
                        marginBottom: 20
                    }}
                >
                    <div
                        style={{
                            fontSize: 16,
                            marginBottom: 20
                        }}
                    >
                        <span>{`Apply the following protocols to policy `}</span>
                        <b id='qos-policy-edit-name'>{policy.policyName}</b>
                        <span>:</span>
                    </div>

                    <div
                        style={{
                            position: 'relative',
                            width: '100%',
                            marginTop: 20,
                            marginBottom: 20
                        }}
                    >
                        <Select
                            name='protocols'
                            isMulti
                            defaultValue={getDefaultProtocolOptions()}
                            options={getProtocolOptions()}
                            placeholder="Select Protocols"
                            id='qos-policy-add-protocol-select'
                            classNamePrefix='qos-policy-add-protocol-select'
                            onChange={(options)=>{
                                let protocols = [];
                                _.forEach(options, (option)=>{
                                    protocols.push({
                                        id: option.value,
                                        appName: option.name
                                    });
                                });

                                setState({
                                    ...policy, 
                                    protocols: protocols
                                });
                            }}
                        />
                    </div>
                </DialogContent>
                <DialogActions>
                    <Button 
                        id='qos-scope-edit-delete'
                        onClick={deletePolicy}
                        variant='outlined'
                        color='secondary'
                        style={{
                            marginRight: 'auto'
                        }}
                    >
                        Delete
                    </Button>
                    <Button id='qos-scope-edit-cancel' onClick={handleClose} color='default'>
                        Cancel
                    </Button>
                    <Button id='qos-scope-edit-submit' onClick={handleSubmit} color='primary'>
                        Apply
                    </Button>
                </DialogActions>
            </Dialog>
        </div>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//
