//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableHead from '@material-ui/core/TableHead';
import TableRow from '@material-ui/core/TableRow';
import { makeStyles } from '@material-ui/core/styles';
import _ from 'lodash';
import Edit from './Edit';

//---------------------------------------------------------------------------//

const useStyles = makeStyles({
    table: {
        minWidth: 650,
    },
    tableCell: {
        textAlign: 'left',
        whiteSpace: 'nowrap'
    }
});

const tableCellList = (values) => {
    let result = [];
    _.forEach(values, (item, index)=>{
        result.push(
            <span
                key={`qos-scope-device-${index}`}
                style={{
                    display:'block'
                }}
            >
                {item.appName}
            </span>
        );
    })
    return result;
};

const Items = (props) => {
    const rows = [];
    const classes = useStyles();
    _.forEach(props.state.qos.policies, (item, index) => {
        rows.push(
            <TableRow 
                key={index}
                hover={true}
            >
                <TableCell 
                    size='small'
                    padding='checkbox'
                >
                    <Edit {...props} policy={item} index={index} />
                </TableCell>
                <TableCell className={classes.tableCell} size='small' id={`qos-policy-tablecell-policyName-${index}`}>{item.policyName}</TableCell>
                <TableCell className={classes.tableCell} size='small' id={`qos-policy-tablecell-policyActionProperty-${index}`}>{item.policyActionProperty.relevanceLevel}</TableCell>
                <TableCell className={classes.tableCell} size='small' id={`qos-policy-tablecell-type-${index}`}>{tableCellList(item.policyResource.applications)}</TableCell>
                <TableCell className={classes.tableCell} size='small' id={`qos-policy-tablecell-policyScope-${index}`}>{item.policyScope}</TableCell>
            </TableRow>
        )
    });
    return rows
};

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    const classes = useStyles();
    return (
        <div style={{overflow:'auto'}}>
            <Table >
                <TableHead>
                    <TableRow>
                        <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}></TableCell>
                        <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}>Policy Name</TableCell>
                        <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}>Relevance Level</TableCell>
                        <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}>Protocol</TableCell>
                        <TableCell className={classes.tableCell} size='small' style={{ fontWeight:'bold' }}>Scope</TableCell>
                    </TableRow>
                </TableHead>
                <TableBody>
                    { <Items {...props}/> }
                </TableBody>
            </Table>
        </div>
    );
};


//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//