//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Button from '@material-ui/core/Button';
import { faPlus } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { makeStyles } from '@material-ui/core/styles';
import TextField from '@material-ui/core/TextField';
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogTitle from '@material-ui/core/DialogTitle';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'

//---------------------------------------------------------------------------//

library.add(faPlus);

const useStyles = makeStyles(theme => ({
    button: {
        position: 'relative',
        top: 10,
        right: 5
    },
}));

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    const classes = useStyles();
    const [open, setOpen] = React.useState(false);
    
    const [policyTag, setState] = React.useState({
        'name': '',
    });

    const handleClickOpen = () => {
        setOpen(true);
        setState({name:''})
    };

    const handleClose = () => {
        setOpen(false);
    };

    const handleSubmit = () => {
        setOpen(false);
        props.dispatchUtil.addPolicyTag({
            policyTag: policyTag.name
        });
    };
    
    const handleChange = (event, key) => {
        let config = { ...policyTag };
        let value = event.target.value;
        config[key] = value;
        setState(config);
    };

    return (
        <div>
            <Button    
                id='qos-scope-add-button'
                className={classes.button} 
                variant='contained' 
                color='primary' 
                size='small'          
                onClick={handleClickOpen}       
                style={{
                    position: 'relative',
                    top: 5
                }}
            >
                <FontAwesomeIcon icon='plus' size='1x' fixedWidth />
                <span style={{marginLeft:5}}>
                    Scope
                </span>
            </Button>
            <Dialog 
                open={open} 
                onClose={handleClose} 
                fullWidth
                maxWidth='xs'
            >
                <DialogTitle id='qos-scope-add'>New Scope</DialogTitle>
                <DialogContent>
                    <TextField
                        margin='dense'
                        id='qos-scope-add-name'
                        label='Name'
                        type='text'
                        value={policyTag.name}
                        onChange={event => handleChange(event, 'name')}
                        fullWidth
                    />
                </DialogContent>
                <DialogActions>
                <Button id='qos-scope-add-cancel' onClick={handleClose} color='default'>
                    Cancel
                </Button>
                <Button id='qos-scope-add-submit' onClick={handleSubmit} color='primary'>
                    Add
                </Button>
                </DialogActions>
            </Dialog>
        </div>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//
