//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Button from '@material-ui/core/Button';
import { faCog } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogTitle from '@material-ui/core/DialogTitle';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import Select from 'react-select';
import { makeStyles } from '@material-ui/core/styles';
import _ from 'lodash';

//---------------------------------------------------------------------------//

library.add(faCog);

const useStyles = makeStyles(theme => ({
    root: {
        overflow: 'visible'
    }
}));

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    const classes = useStyles();

    const [open, setOpen] = React.useState(false);

    let initialConfig = {
        policyTag: props.scope.policyTag,
        networkDevices: props.scope.networkDevices?props.scope.networkDevices:[]
    };

    const [scope, setState] = React.useState(initialConfig);

    const handleClickOpen = () => {
        setOpen(true);
        setState(initialConfig)
    };

    const handleClose = () => {
        setOpen(false);
    };

    const deletePolicyTagAssociationAndPolicy= () => {
        setOpen(false);
        props.dispatchUtil.deletePolicyTagAssociationAndPolicy(scope.policyTag);
    };

    const handleSubmit = () => {

        let networkDevices = [];

        _.forEach(scope.networkDevices, (id)=>{
            let device = _.find(props.state.inventory, {id:id});
            networkDevices.push({
                deviceId: device.id,
            });
        })

        props.dispatchUtil.updatePolicyTagAssociation({
            policyTag: scope.policyTag,
            networkDevices: networkDevices
        });
        setOpen(false);
    };

    const getDeviceOptions = () => {
        let result = [];
        _.forEach(props.state.inventory, (device)=>{
            let hostname = device.hostname?device.hostname:'';
            let managementIpAddress = device.managementIpAddress?device.managementIpAddress:'';
            result.push({
                label: `${hostname} - ${managementIpAddress}`,
                value: device.id
            });
        });
        return result;
    };

    const getDefaultDeviceOptions = () => {
        let result = [];
        _.forEach(scope.networkDevices, (device)=>{
            result.push({
                label: `${device.deviceName} - ${device.deviceIp}`,
                value: device.deviceId
            });
        });
        return result;
    };

    return (
        <div>
            <Button
                id={`qos-scope-edit-button-${props.index}`}
                variant='contained' 
                color='default' 
                size='small'
                fullWidth={true}  
                onClick={handleClickOpen}
                style={{
                    maxWidth: 25,
                    maxHeight: 25, 
                    minWidth: 25, 
                    minHeight: 25
                }}
            >
                <FontAwesomeIcon 
                    icon='cog' 
                    size='1x' 
                />
            </Button>

            <Dialog 
                open={open} 
                onClose={handleClose} 
                scroll= 'body'
                classes={{ 
                    paperScrollBody: classes.root 
                }}
                maxWidth='md'
                fullWidth
            >
                <DialogTitle>Edit Scope</DialogTitle>
                <DialogContent
                    className={classes.root}
                    style={{
                        marginBottom: 20
                    }}
                >
                    <div
                        style={{
                            fontSize: 16,
                            marginBottom: 20
                        }}
                    >
                        <span>{`Associate scope `}</span>
                        <b id='qos-scope-edit-name'>{scope.policyTag}</b>
                        <span>{' with the following network devices:'}</span>
                    </div>

                    <div
                        style={{
                            position: 'relative',
                            width: '100%',
                        }}
                    >
                        <Select
                            isMulti
                            name='scopes'
                            defaultValue={getDefaultDeviceOptions()}
                            options={getDeviceOptions()}
                            placeholder="Select Network Devices"
                            id='qos-scope-edit-device-select'
                            classNamePrefix='qos-scope-edit-device-select'
                            onChange={(option)=>{
                                let state = { ...scope };
                                let devices = [];
                                _.forEach(option, (device)=>{
                                    devices.push(device.value)
                                });
                                state['networkDevices'] = devices;
                                setState(state);
                            }}
                        />
                    </div>
                </DialogContent>
                <DialogActions>
                    <Button
                        id='qos-scope-edit-delete'
                        onClick={deletePolicyTagAssociationAndPolicy}
                        variant='outlined'
                        color='secondary'
                        style={{
                            marginRight: 'auto'
                        }}
                    >
                        Delete
                    </Button>
                    <Button id='qos-scope-edit-cancel' onClick={handleClose} color='default'>
                        Cancel
                    </Button>
                    <Button id='qos-scope-edit-submit' onClick={handleSubmit} color='primary'>
                        Apply
                    </Button>
                </DialogActions>
            </Dialog>
        </div>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//
