//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React, { useEffect } from 'react';
import Grid from '@material-ui/core/Grid';
import { connect } from 'react-redux';
import Container from '@material-ui/core/Container';

import Policy from './components/Policy';
import Scope from './components/Scope';
import AppBar from '@material-ui/core/AppBar';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import { useHistory } from 'react-router-dom';

import CommonApi from '../../Api';

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    let history = useHistory();
    const navigate = (view) => {
        history.push('/' + view);
    };

    useEffect(() => {
        props.dispatchUtil.getFilterApplication('');
        props.dispatchUtil.getFilterPolicies('');
        props.dispatchUtil.getNetworkDevices();
        props.dispatchUtil.getPolicyTagAssociation();

        // eslint-disable-next-line
    },[]); 

    return (
        <div>
            <AppBar
                position='static'
                elevation={0}
            >
                <Container maxWidth='xl'>
                    <Tabs value={0} >
                        <Tab id='navbar-tab-qos' onClick={navigate.bind(this, 'qos')} label='QOS' />
                        <Tab id='navbar-tab-setting' onClick={navigate.bind(this, 'setting')} label='Network Settings' />
                    </Tabs>

                </Container>

            </AppBar>
            <Container maxWidth='xl'>
                <Grid 
                    container
                    direction='row'
                    justify='center'
                    spacing={3}
                >
                    <Grid item xl={4} lg={5} style={{width:'100%'}}>
                        <Scope {...props} />
                    </Grid>
                    <Grid item xl={8} lg={7} style={{width:'100%'}}>
                        <Policy {...props} />
                    </Grid>
                </Grid>
                <p>
                <strong>Note</strong>: This functionality is only supported on devices running IOS-XE OS
                </p>
            </Container>
        </div>

    );
};

//---------------------------------------------------------------------------//
// Redux Actions
//---------------------------------------------------------------------------//

const mapStateToProps = state => ({
    state: {
        qos: state.qos,
        inventory: state.inventory
    }
});

const mapDispatchToProps = dispatch => ({
    dispatchUtil: {

        getFilterApplication: (name) => {
            CommonApi.Qos.getFilterApplication(dispatch, name);
        },

        getFilterPolicies: (query) => {
            CommonApi.Qos.getFilterPolicies(dispatch, query);
        },

        addPolicy: (policy) => {
            CommonApi.Qos.addPolicy(dispatch, policy);
        },

        deletePolicy: (id) => {
            CommonApi.Qos.deletePolicy(dispatch, id);
        },

        addPolicyTag: (config) => {
            CommonApi.Qos.addPolicyTag(dispatch, config);
        },

        getPolicyTags: () => {
            CommonApi.Qos.getPolicyTags(dispatch);
        },

        getNetworkDevices: () => {
            CommonApi.Qos.getNetworkDevices(dispatch);
        },

        deletePolicyTag: (id) => {
            CommonApi.Qos.deletePolicyTag(dispatch, id);
        },

        deletePolicyTagAssociationAndPolicy: (policyTag) => {
            CommonApi.Qos.deletePolicyTagAssociationAndPolicy(dispatch, policyTag);
        },

        deletePolicyTagAssociation: (policyTag, deviceId) => {
            CommonApi.Qos.deletePolicyTagAssociation(dispatch, policyTag, deviceId);
        },

        addPolicyTagAssociation: (scope) => {
            CommonApi.Qos.addPolicyTagAssociation(dispatch, scope);
        },

        updatePolicyTagAssociation: (config) => {
            CommonApi.Qos.updatePolicyTagAssociation(dispatch, config);
        },

        getPolicyTagAssociation: (query) => {
            CommonApi.Qos.getPolicyTagAssociation(dispatch, query);
        },

        updatePolicy: (config) => {
            CommonApi.Qos.updatePolicy(dispatch, config);
        }
    }
});

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default connect(
    mapStateToProps,
    mapDispatchToProps
)(Component);

//---------------------------------------------------------------------------//