//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Button from '@material-ui/core/Button';
import { faCloudUploadAlt } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogTitle from '@material-ui/core/DialogTitle';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableRow from '@material-ui/core/TableRow';
import _ from 'lodash';

//---------------------------------------------------------------------------//

library.add(faCloudUploadAlt);

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const [open, setOpen] = React.useState(false);

    const handleClickOpen = () => {
        setOpen(true);
    };

    const handleClose = () => {
        setOpen(false);
    };

    const handleSubmit = () => {
        props.dispatchUtil.edit(props.state);
        setOpen(false);
    };

    const infoRowTitle = (value) => {
        return(
            <TableRow>
                <TableCell colSpan={2} size='small' style={{ fontWeight:'bold', textAlign:'left', background:'white', color:'black', borderTop:'grey 1.5px solid'}}>{value}</TableCell>
            </TableRow>
        );
    };

    const infoRow = (label, group, key) => {

        let value = props.state[group][key];
        if(value===''){
            value = '-';
        }

        return(
            <TableRow hover={true}>
                <TableCell id={`settings-push-tablecell-label-${group}-${key}`} align='right' size='small' style={{fontWeight:'bold',width:'50%'}}>{label}</TableCell>
                <TableCell id={`settings-push-tablecell-value-${group}-${key}`} align='left' size='small'>{value}</TableCell>
            </TableRow>
        );
    };

    const getSyslog = () => {
        let result = [];
        _.forEach(props.state.syslog, (syslog, index)=>{
            let value = syslog.serverIp;
            result.push(
                <TableRow 
                    key={`syslog-${index}`}
                    hover={true}
                >
                    <TableCell id={`settings-push-tablecell-label-syslog-${index}`} align='right' size='small' style={{fontWeight:'bold',width:'50%'}}>{`Server Ip (${index+1})`}</TableCell>
                    <TableCell id={`settings-push-tablecell-value-syslog-${index}`} align='left' size='small'>{value}</TableCell>
                </TableRow>
            );
        })

        if(result.length<1){
            result.push(
                <TableRow
                    key='syslog-0'
                    hover={true}
                >
                    <TableCell id={`settings-push-tablecell-label-syslog-0`} align='right' size='small' style={{fontWeight:'bold',width:'50%'}}>Server Ip</TableCell>
                    <TableCell id={`settings-push-tablecell-value-syslog-0`} align='left' size='small'>-</TableCell>
                </TableRow>
            );
        }

        return result;
    };

    return (
        <>
            <Button    
                id='settings-push-button'
                variant='contained' 
                color='primary' 
                size='small'          
                onClick={handleClickOpen}       
                style={{
                    position: 'relative',
                    top: 5,
                    marginRight: '5px'
                }}
            >
                <FontAwesomeIcon icon='cloud-upload-alt' size='1x' fixedWidth />
                <span style={{marginLeft:5}}>
                    Push Config
                </span>
            </Button>

            <Dialog open={open} onClose={handleClose} aria-labelledby='Confirmation Dialog'>
                <DialogTitle id='settings-push-title'>Push All Network Settings </DialogTitle>
                <DialogContent>
                    <span style={{display:'block',fontSize:18}}>The following configurations will be saved and applied to the network wide settings. </span>
                    <span style={{display:'block',fontSize:16,color:'blue', marginTop:10}}> Note: If you have not saved yet, please do so first. </span>
                    <Table style={{marginTop: 25, marginBottom: 25}}>
                        <TableBody>
                            {infoRowTitle('AAA')}
                            {infoRow('Ip Address', 'aaa', 'ipAddress')}
                            {infoRow('Key', 'aaa', 'key')}
                            {infoRowTitle('DNS')}
                            {infoRow('Domain Name', 'dns', 'ipAddress')}
                            {infoRow('Ip Address', 'dns', 'name')}
                            {infoRowTitle('NETFLOW')}
                            {infoRow('Reflection Ip', 'netflow', 'port')}
                            {infoRow('Port Number', 'netflow', 'reflectionIp')}
                            {infoRowTitle('NTP')}
                            {infoRow('Server Ip', 'ntp', 'serverIp')}
                            {infoRowTitle('SYSLOG')}
                            {getSyslog()}
                        </TableBody>
                    </Table>
                </DialogContent>
                <DialogActions>
                <Button id='settings-push-cancel' onClick={handleClose} color='default'>
                    Cancel
                </Button>
                <Button id='settings-push-submit' onClick={handleSubmit} color='primary'>
                    OKAY
                </Button>
                </DialogActions>
            </Dialog>
        </>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//