//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import { makeStyles } from '@material-ui/core/styles';
import TextField from '@material-ui/core/TextField';
import Button from '@material-ui/core/Button';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faMinus, faPlus } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import Save from './Save';

//---------------------------------------------------------------------------//

library.add(faMinus, faPlus);
const useStyles = makeStyles(theme => ({
    root:{
        minWidth: 50,
        maxWidth: 50
    }
}));

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const classes = useStyles();

    const getInitState = () => {
        if(props.settings.syslog.length){
            return props.settings.syslog;
        }else{
            return [{
                serverIp: ''
            }]
        }
    };

    const [syslogs, setState] = React.useState(getInitState());

    // Use time as state to trigger render when syslog change.
    // eslint-disable-next-line
    const [date, setDate] = React.useState(Date());

    const addRow = ()=>{
        let newState = syslogs;
        newState.push({
            serverIp: ''
        })
        setState(newState);
        setDate(new Date());
    };

    const removeRow = ()=>{
        let newState = syslogs;
        if(newState.length>1){
            newState.pop();
            setState(newState);
            setDate(new Date());
        }
    };

    const handleChange = (event, index) => {
        let value = event.target.value;
        syslogs[index] = {
            serverIp: value
        };
        setState(syslogs);
        setDate(new Date());
    };

    const createField = (item, index)=>{
        return(
            <TextField 
                style={{marginBottom: 10}}
                id={`setting-syslog-serverIp_${index}`}
                label={`Server Ip (${index + 1})`}
                fullWidth
                value={item.serverIp}
                key={index}
                onChange={(event) => {
                    let id = event.target.id;
                    let index = parseInt(id.split('_')[1]);
                    handleChange(event, index);
                }}
            />
        )
    };

    const rows = () => {
        let result = [];
        syslogs.forEach(function(item, index){
            result.push(createField(item, index));
        });
        return result;
    };

    return (
        <>
            <form>
                <div
                    style={{
                        marginBottom:20
                    }}
                >
                    <Button
                        classes={{
                            root: classes.root
                        }}
                        variant='contained' 
                        color='default' 
                        id='setting-syslog-button-remove'
                        onClick={removeRow}
                        size='small'
                        style={{
                            marginRight:5
                        }}
                    >
                        <FontAwesomeIcon 
                            icon='minus' 
                            size='1x' 
                        />
                    </Button>
                    <Button
                        variant='contained' 
                        color='default' 
                        id='setting-syslog-button-add'
                        onClick={addRow}
                        size='small'
                        classes={{
                            root: classes.root
                        }}
                    >
                        <FontAwesomeIcon 
                            icon='plus' 
                            size='1x' 
                        />
                    </Button>
                </div>
                {rows()}
            </form>
            <Save {...props} payload={{syslog: syslogs}}/>
        </>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//