//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React, { useEffect } from 'react';
import { makeStyles } from '@material-ui/core/styles';
import Container from '@material-ui/core/Container';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import Typography from '@material-ui/core/Typography';
import Box from '@material-ui/core/Box';
import Card from '@material-ui/core/Card';
import CardHeader from '@material-ui/core/CardHeader';
import CardContent from '@material-ui/core/CardContent';
import AppBar from '@material-ui/core/AppBar';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faCogs } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { connect } from 'react-redux';
import { useHistory } from 'react-router-dom';

import CommonApi from '../../Api';

import Netflow from './components/Netflow';
import Dns from './components/Dns';
import AAA from './components/AAA';
import Syslog from './components/Syslog';
import Ntp from './components/Ntp';
import PushConfigs from './components/PushConfigs';

import Loading from '../../components/Loading';

//---------------------------------------------------------------------------//

library.add(faCogs);

const useStyles = makeStyles(theme => ({
    container:{
        marginTop: 15,
    },
    card:{
        paddingTop: 10,
        paddingBottom: 20
    },
    cardContent:{
        flexGrow: 1,
        backgroundColor: theme.palette.background.paper,
        display: 'flex'
    },
    tabs: {
        borderRight: `1px solid ${theme.palette.divider}`,
        minWidth: 100
    },
}));

function TabPanel(props) {
    const { children, value, index, ...other } = props;

    return (
        <Typography
            component='div'
            role='tabpanel'
            hidden={value !== index}
            aria-labelledby={`vertical-tab-${index}`}
            {...other}
        >
            {value === index && <Box p={3}>{children}</Box>}
        </Typography>
    );
}

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const classes = useStyles();
    const [value, setValue] = React.useState(0);
    const [fetchCompleted, setFetchCompleted] = React.useState(false);
    let history = useHistory();

    const handleChange = (event, newValue) => {
        setValue(newValue);
    };

    const fetchCallback =() => {
        setFetchCompleted(true);
    }

    useEffect(() => {
        props.dispatchUtil.fetch(fetchCallback);
    },[props.dispatchUtil]); 

    const navigate = (view) => {
        history.push('/' + view);
    };

    if(fetchCompleted){
        return (
            <>
                <AppBar
                    position='static'
                    elevation={0}
                >
                    <Container maxWidth='xl'>
                        <Tabs value={1} >
                            <Tab id='navbar-tab-qos' onClick={navigate.bind(this, 'qos')} label='QOS' />
                            <Tab id='navbar-tab-setting' onClick={navigate.bind(this, 'setting')} label='Network Settings' />
                        </Tabs>
                    </Container>
                </AppBar>

                <Container className={classes.container} maxWidth='md'>
                    <Card className={classes.card}>
                        <CardHeader 
                            avatar={
                                <FontAwesomeIcon icon='cogs' size='lg' fixedWidth />
                            }
                            title='Network Settings'
                            action={
                                <PushConfigs {...props} />
                            }
                            style={{
                                color: '#495057',
                                borderBottom: '1px #dde0e6 solid',
                                paddingTop: 5,
                                paddingBottom: 10,
                                paddingLeft: 15
                            }}
                        >
                        </CardHeader>
                        <CardContent className={classes.cardContent}>
                            <Tabs
                                orientation='vertical'
                                variant='scrollable'
                                value={value}
                                onChange={handleChange}
                                aria-label='Vertical tabs example'
                                className={classes.tabs}
                            >
                                <Tab id='setting-tab-aaa' label='AAA' />
                                <Tab id='setting-tab-dns' label='DNS' />
                                <Tab id='setting-tab-netflow' label='Net Flow' />
                                <Tab id='setting-tab-ntp' label='Ntp' />
                                <Tab id='setting-tab-syslog' label='Syslog' />
                            </Tabs>
                            <TabPanel value={value} index={0} style={{width:'100%'}}>
                                <AAA settings={props.state} dispatchUtil={props.dispatchUtil} />
                            </TabPanel>
                            <TabPanel value={value} index={1} style={{width:'100%'}}>
                                <Dns settings={props.state} dispatchUtil={props.dispatchUtil} />
                            </TabPanel>
                            <TabPanel value={value} index={2} style={{width:'100%'}}>
                                <Netflow settings={props.state} dispatchUtil={props.dispatchUtil} />
                            </TabPanel>
                            <TabPanel value={value} index={3} style={{width:'100%'}}>
                                <Ntp settings={props.state} dispatchUtil={props.dispatchUtil} />
                            </TabPanel>
                            <TabPanel value={value} index={4} style={{width:'100%'}}>
                                <Syslog settings={props.state} dispatchUtil={props.dispatchUtil} />
                            </TabPanel>
                        </CardContent>
                    </Card>
                <p>
                <strong>Note</strong>: This functionality is only supported on devices running IOS-XE OS and Switch 2960-24TT
                </p>        
                </Container>
            </>
        );
    }else{
        return <Loading />;
    }

};

//---------------------------------------------------------------------------//
// Redux Actions
//---------------------------------------------------------------------------//

const mapStateToProps = state => ({
    state: state.setting
});

const mapDispatchToProps = dispatch => ({

    dispatchUtil: {

        fetch: (fetchCallback) => {
            CommonApi.Settings.getNetworkWideSetting(dispatch, fetchCallback);
        },

        edit: (settings) => {
            CommonApi.Settings.updateNetworkWideSetting(dispatch, settings);
        },

        changeSettings: (settings) => {
            dispatch({
                type: 'SETTINGS_CHANGE',
                payload: {
                    settings: {...settings}
                }
            });
        }
    }
});

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default connect(
    mapStateToProps,
    mapDispatchToProps
)(Component);

//---------------------------------------------------------------------------//