//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Card from '@material-ui/core/Card';
import CardHeader from '@material-ui/core/CardHeader';
import CardContent from '@material-ui/core/CardContent';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faLink} from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableRow from '@material-ui/core/TableRow';
import TableHead from '@material-ui/core/TableHead';

//---------------------------------------------------------------------------//

library.add(faLink);

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const infoRow = (label, key, value) => {

        let sourceValue = props.state.topology.detail.source[key];
        let targetValue = props.state.topology.detail.target[key];

        if(key==='lastUpdateTime'){
            sourceValue += ' seconds';
            targetValue += ' seconds';
        }

        return(
            <TableRow 
                key={key}
                hover={true}
            >
                <TableCell size='small' style={{fontWeight:'bold', textAlign:'right'}}>{label}</TableCell>
                <TableCell size='small' style={{ textAlign:'center'}} id={`topology-detail-source-info-${key}`}>{sourceValue}</TableCell>
                <TableCell size='small' style={{ textAlign:'center'}} id={`topology-detail-target-info-${key}`}>{targetValue}</TableCell>
            </TableRow>
        );
    };

    let link = props.state.topology.detail.link;

    return (
        <div>
            <Card>
                <CardHeader
                    avatar={
                        <FontAwesomeIcon icon='link' size='lg' fixedWidth />
                    }
                    title='Link'
                    style={{
                        borderBottom: '1px #dde0e6 solid',
                        color: '#495057'
                    }}
                />
                <CardContent>
                    <div style={{display:'block', marginBottom:10}}>
                        <b>{'ID: '}</b><span>{link.id}</span>
                    </div>
                    <div>
                        <b>{'Link Status: '}</b><span>{link.linkStatus}</span>
                    </div>
                    <Table>
                        <TableHead>
                            <TableRow>
                                <TableCell></TableCell>
                                <TableCell size='small' style={{ fontWeight:'bold', textAlign:'center' }}>Starting Port</TableCell>
                                <TableCell size='small' style={{ fontWeight:'bold', textAlign:'center' }}>Ending Port</TableCell>
                            </TableRow>
                        </TableHead>
                        <TableBody>
                            <TableRow 
                                hover={true}
                            >
                                <TableCell size='small' style={{fontWeight:'bold', textAlign:'right'}}>Name</TableCell>
                                <TableCell size='small' style={{ textAlign:'center'}} id={`topology-detail-source-info-`}>{link.startPortName}</TableCell>
                                <TableCell size='small' style={{ textAlign:'center'}} id={`topology-detail-source-info-`}>{link.endPortName}</TableCell>
                            </TableRow>
                            <TableRow 
                                hover={true}
                            >
                                <TableCell size='small' style={{fontWeight:'bold', textAlign:'right'}}>Speed</TableCell>
                                <TableCell size='small' style={{ textAlign:'center'}} id={`topology-detail-source-info-`}>{link.startPortSpeed}</TableCell>
                                <TableCell size='small' style={{ textAlign:'center'}} id={`topology-detail-source-info-`}>{link.endPortSpeed}</TableCell>
                            </TableRow>
                        </TableBody>
                    </Table>
                </CardContent>
            </Card>
            <Card
                style={{
                    marginTop: 20
                }}
            >
                <CardContent
                    style={{
                        overflow:'auto'
                    }}
                >
                    <Table>
                        <TableHead>
                            <TableRow>
                                <TableCell></TableCell>
                                <TableCell size='small' style={{ fontWeight:'bold', textAlign:'center' }}>Source Device</TableCell>
                                <TableCell size='small' style={{ fontWeight:'bold', textAlign:'center' }}>Target Device</TableCell>
                            </TableRow>
                        </TableHead>
                        <TableBody>
                            {infoRow('Collection Status', 'collectionStatus')}
                            {infoRow('Error Description', 'errorDescription')}
                            {infoRow('Hostname', 'hostname')}
                            {infoRow('ID', 'id')}
                            {infoRow('Interface Count', 'interfaceCount')}
                            {infoRow('Inventory Status Detail', 'inventoryStatusDetail')}
                            {infoRow('IOS Version', 'iosVersion')}
                            {infoRow('Last Update Time', 'lastUpdateTime')}
                            {infoRow('Last Updated', 'lastUpdated')}
                            {infoRow('MAC Address', 'macAddress')}
                            {infoRow('Management IP Address', 'managementIpAddress')}
                            {infoRow('Platform ID', 'platformId')}
                            {infoRow('Product ID', 'productId')}
                            {infoRow('Reachability Failure Reason', 'reachabilityFailureReason')}
                            {infoRow('Reachability Status', 'reachabilityStatus')}
                            {infoRow('Serial Number', 'serialNumber')}
                            {infoRow('Type', 'type')}
                            {infoRow('UpTime', 'upTime')}
                        </TableBody>
                    </Table>
                </CardContent>
            </Card>

        </div>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//