//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Card from '@material-ui/core/Card';
import CardHeader from '@material-ui/core/CardHeader';
import CardContent from '@material-ui/core/CardContent';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faHdd} from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableRow from '@material-ui/core/TableRow';
import TableHead from '@material-ui/core/TableHead';

//---------------------------------------------------------------------------//

library.add(faHdd);

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const infoRow = (label, key, setValue) => {
        
        let value = props.state.topology.detail.device[key];

        if(setValue){
            value = setValue;
        }

        return(
            <TableRow 
                key={key}
                hover={true}
            >
                <TableCell size='small' style={{fontWeight:'bold', textAlign:'right'}}>{label}</TableCell>
                <TableCell size='small' style={{ textAlign:'left'}} id={`topology-detail-info-${key}`}>{value}</TableCell>
            </TableRow>
        );
    };

    let node = props.state.topology.detail.node;

    return (
        <Card>
            <CardHeader
                avatar={
                    <FontAwesomeIcon icon='hdd' size='lg' fixedWidth />
                }
                title='Node'
                style={{
                    borderBottom: '1px #dde0e6 solid',
                    color: '#495057'
                }}
            />
            <CardContent>
                <div style={{display:'block', marginBottom:10}}>
                    <b>{'ID: '}</b><span>{node.id}</span>
                </div>
                <div style={{display:'block'}}>
                    <b>{'XY Coordinate: '}</b><span>{`(${node.x}, ${node.y})`}</span>
                </div>
                <Table>
                    <TableHead>
                        <TableRow>
                            <TableCell colSpan={2} size='small' style={{ fontWeight:'bold', textAlign:'center', color:'grey'}}>Device Detail</TableCell>
                        </TableRow>
                    </TableHead>
                    <TableBody>
                        {infoRow('Collection Status', 'collectionStatus')}
                        {infoRow('Error Description', 'errorDescription')}
                        {infoRow('Hostname', 'hostname')}
                        {infoRow('ID', 'id')}
                        {infoRow('Interface Count', 'interfaceCount')}
                        {infoRow('Inventory Status Detail', 'inventoryStatusDetail')}
                        {infoRow('IOS Version', 'iosVersion')}
                        {infoRow('Last Update Time', 'lastUpdateTime', `${props.state.topology.detail.device['lastUpdateTime']} seconds`)}
                        {infoRow('Last Updated', 'lastUpdated')}
                        {infoRow('MAC Address', 'macAddress')}
                        {infoRow('Management IP Address', 'managementIpAddress')}
                        {infoRow('Platform ID', 'platformId')}
                        {infoRow('Product ID', 'productId')}
                        {infoRow('Reachability Failure Reason', 'reachabilityFailureReason')}
                        {infoRow('Reachability Status', 'reachabilityStatus')}
                        {infoRow('Serial Number', 'serialNumber')}
                        {infoRow('Type', 'type')}
                        {infoRow('UpTime', 'upTime')}
                    </TableBody>
                </Table>
            </CardContent>
        </Card>
    );
};

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//