//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import { Graph } from 'react-d3-graph';
import _ from 'lodash';
import * as d3 from 'd3';
import util from '../../../utils';

//---------------------------------------------------------------------------//

const Width = 1800;
const Height = 700;

const config = {
    "automaticRearrangeAfterDropNode": false,
    "collapsible": false,
    "directed": false,
    "focusAnimationDuration": 0.75,
    "focusZoom": 1,
    "height": Height,
    "highlightDegree": 1,
    "highlightOpacity": 1,
    "linkHighlightBehavior": true,
    "maxZoom": 1,
    "minZoom": 0.6,
    "nodeHighlightBehavior": true,
    "panAndZoom": false,
    "staticGraph": false,
    "staticGraphWithDragAndDrop": true,
    "width": Width,
    
    "d3": {
        "backgroundColor":'red',
        "alphaTarget": 0.05,
        "gravity": -400,
        "linkLength": 180,
        "linkStrength": 1,
        "disableLinkForce": true
    },
    "node": {
        "fontColor": "black",
        "fontSize": 16,
        "highlightFontSize": 16,
        "highlightFontWeight": "bold",
        "highlightStrokeColor": "SAME",
        "highlightStrokeWidth": "SAME",
        "mouseCursor": "pointer",
        "renderLabel": true,
        "strokeColor": "none",
        labelProperty: (node) => {
            return node.device.hostname;
        }, 
    },
    "link": {
        "highlightColor": "orange",
        "highlightFontSize": 8,
        "highlightFontWeight": "bold",
        "labelProperty": "label",
        "mouseCursor": "pointer",
        "opacity": 1,
        "renderLabel": true,
        "semanticStrokeWidth": true,
        "strokeWidth": 3,
        "markerHeight": 6,
        "markerWidth": 6
    }
};

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    let graphData = {
        nodes: [],
        links: []
    }

    const myScale = d3.scaleLinear().domain([0, Width]).range([15, Height]);
    
    _.forEach(props.state.topology.nodes, (node)=>{
        let device = _.find(props.state.inventory, {id: node.id});
        if(device){
            let image = _.find(util.images, {type: device.type});

            let x = _.round(myScale(node.x), 0);
            let y = _.round(myScale(node.y), 0);

            graphData.nodes.push({
                id: node.id,
                device: device,
                x: x,
                y: y,
                svg: `images/${image?image.name:util.images[0].name}`,
                size: 400
            });
        }
    });

    _.forEach(props.state.topology.links, (link)=>{
        graphData.links.push({
            id: link.id,
            source: link.source, 
            target: link.target,
            color: link.linkStatus==="1"?'green':'red',
            fontSize: 18,
        });
    });
    
    const onClickNode = function(nodeId) {
        let device = _.find(props.state.inventory, {id: nodeId});
        let node = _.find(props.state.topology.nodes, {id: nodeId});
        props.dispatchUtil.detail({
            viewType: 'node',
            node: node,
            device: device
        });
    };
    
    const onClickLink = function(source, target) {
        let sourceDevice = _.find(props.state.inventory, {id: source});
        let targetDevice = _.find(props.state.inventory, {id: target});
        let link = _.find(props.state.topology.links, {
            source: source,
            target: target
        });
        props.dispatchUtil.detail({
            viewType: 'link',
            link: link,
            source: sourceDevice,
            target: targetDevice
        });
    };

    if(graphData.nodes.length){

        // Set key on DIV to force component to re-render.
        return (
            <div
                key={props.state.topology.lastUpdated}
                style={{
                    overflow:'auto'
                }}
            >
                <Graph
                    id='typology-graph'
                    data={graphData}
                    config={config}
                    onClickNode={onClickNode}
                    onClickLink={onClickLink}
                >
                </Graph>
            </div>
        );
    }else{
        return null;
    }

}

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component

//---------------------------------------------------------------------------//