//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import Grid from '@material-ui/core/Grid';
import { connect } from 'react-redux';
import Container from '@material-ui/core/Container';
import useDeepCompareEffect from 'use-deep-compare-effect';
import Graph from './components/Graph';
import Detail from './components/Detail';

import Card from '@material-ui/core/Card';
import CardHeader from '@material-ui/core/CardHeader';
import CardContent from '@material-ui/core/CardContent';
import AppBar from '@material-ui/core/AppBar';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import { useHistory } from 'react-router-dom';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faHdd, faNetworkWired } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import CommonApi from '../../Api';
import _ from 'lodash';

//---------------------------------------------------------------------------//

library.add(faHdd, faNetworkWired);

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {
    let history = useHistory();
    const navigate = (view) => {
        history.push('/' + view);
    };
    
    useDeepCompareEffect(() => {
        props.dispatchUtil.lastUpdated();
        props.dispatchUtil.getPhysicalTopology();
        props.dispatchUtil.fetchInventory();
        let timer = setInterval(()=>{
            props.dispatchUtil.getPhysicalTopology();
            props.dispatchUtil.fetchInventory();
        }, 15000);
        
        return () => {
            // Clean up the timer.
            clearTimeout(timer);
        };
        // eslint-disable-next-line
    },[props.state.topology.nodes]); 
    
    return (
        <>
            <AppBar
                position='static'
                elevation={0}
                style={{
                    background: '#1e4471',
                    color:'white',
                    marginBottom: 15
                }}    
            >
                <Container maxWidth='xl'>
                    <Tabs value={2} >
                        <Tab id='navbar-tab-assurance' onClick={navigate.bind(this, 'assurance')} label='Assurance' />
                        <Tab id='navbar-tab-host' onClick={navigate.bind(this, 'host')} label='Hosts' />
                        <Tab id='navbar-tab-topology' onClick={navigate.bind(this, 'topology')} label='Topology' />
                        <Tab id='navbar-tab-pathtrace' onClick={navigate.bind(this, 'pathtrace')}label='Path Trace' />
                    </Tabs>
                </Container>
            </AppBar>
            <Container maxWidth='xl'>
                <Grid 
                    container
                    direction='row'
                    justify='center'
                    alignItems='flex-start'
                    spacing={3}
                >
                    <Grid 
                        item 
                        lg={_.isEmpty(props.state.topology.detail)?12:5}
                        md={_.isEmpty(props.state.topology.detail)?12:6}
                    >
                        <Card>
                            <CardHeader
                                avatar={
                                    <FontAwesomeIcon icon='network-wired' size='lg' fixedWidth />
                                }
                                title='Topology'
                                style={{
                                    borderBottom: '1px #dde0e6 solid',
                                    color: '#495057'
                                }}
                            />
                            <CardContent>
                                <Graph {...props} />
                            </CardContent>
                        </Card>
                    </Grid>
                    <Detail {...props} />
                </Grid>
            </Container>
        </>
    );
};

//---------------------------------------------------------------------------//
// Redux Actions
//---------------------------------------------------------------------------//

const mapStateToProps = state => ({
    state:{
        topology: state.topology,
        inventory: state.inventory
    }
});

const mapDispatchToProps = dispatch => ({
    dispatchUtil: {
        fetchInventory: () => {
            CommonApi.Inventory.getNetworkDevices(dispatch);
        },
        getPhysicalTopology: (id) => {
            CommonApi.Topology.getPhysicalTopology(dispatch);
        },
        detail: (item) => {
            dispatch({
                type: 'TOPOLOGY_DETAIL',
                payload: {
                    detail: item
                }
            });
        },
        lastUpdated: (item) => {
            dispatch({
                type: 'TOPOLOGY_LASTUPDATED',
                payload: {
                    time: new Date().getTime()
                }
            });
        }
    }
});

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default connect(
    mapStateToProps,
    mapDispatchToProps
)(Component);

//---------------------------------------------------------------------------//