//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React from 'react';
import TextField from '@material-ui/core/TextField';
import Button from '@material-ui/core/Button';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableHead from '@material-ui/core/TableHead';
import TableRow from '@material-ui/core/TableRow';
import _ from 'lodash';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faSave } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';
import { toast } from 'react-toastify';
import 'react-toastify/dist/ReactToastify.css';
//---------------------------------------------------------------------------//

library.add(faSave);

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    let username = props.state.app.username;
    let user = {};
    
    _.forEach(props.state.users, (item)=>{
        if(username === item.username){
            user = item;
        }
    });

    const [state, setState] = React.useState({
        username: user.username,
        role: user.authorization[0].role,
        scope: user.authorization[0].scope,
        password: '',
        confirmPassword: ''
    });

    const handleChange = (event, key) => {
        let result = { ...state };
        let value = event.target.value;
        result[key] = value;
        setState(result);
    };
    
    const save = () => {
        toast.configure();
        if(state.password===''){
            toast.error('Password field cannot be empty.', {
                position: toast.POSITION.BOTTOM_CENTER
            });
        }
        else if(state.password !== state.confirmPassword){
            toast.error('Confirmed password does not match.', {
                position: toast.POSITION.BOTTOM_CENTER
            });
        }else{
            let payload = {
                username: state.username,
                password: state.password,
                authorization: [{
                    scope: state.scope,
                    role: state.role
                }]
            }
            props.dispatchUtil.updateUser(payload);
        }
    };

    return (
        <div>

            <Table
                style={{
                    marginBottom: 25
                }}
            >
                <TableHead>
                    <TableRow>
                        <TableCell align='center' size='small' style={{ fontWeight: 'bold' }}>Username</TableCell>
                        <TableCell align='center' size='small' style={{ fontWeight: 'bold' }}>Role</TableCell>
                    </TableRow>
                </TableHead>
                <TableBody>
                    <TableRow>
                        <TableCell id="user-config-username" align='center' size='small'>{state.username}</TableCell>
                        <TableCell id="user-config-role" align='center' size='small'>{state.role}</TableCell>
                    </TableRow>
                </TableBody>
            </Table>

            <form>
                <div>
                    <TextField 
                        id='user-config-form-password' 
                        label='New Password' 
                        fullWidth
                        onChange={event => handleChange(event, 'password')}
                        type='password'
                    />
                </div>
                <div>
                    <TextField 
                        id='user-config-form-confirmPassword' 
                        label='Confirm New Password' 
                        fullWidth
                        onChange={event => handleChange(event, 'confirmPassword')}
                        type='password'
                    />
                </div>
            </form>

            <div
                style={{
                    marginTop: 20
                }}
            >
                <Button 
                    id='user-config-form-submit' 
                    onClick={save} 
                    variant='contained' 
                    color='primary' 
                    style={{
                        float:'right'
                    }}>
                    <FontAwesomeIcon icon='save' size='1x' fixedWidth />
                    <span
                        style={{
                            marginLeft: 5
                        }}
                    >
                        Save
                    </span>
                </Button>
            </div>
        </div>
    );
}

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default Component;

//---------------------------------------------------------------------------//