//---------------------------------------------------------------------------//
// Import
//---------------------------------------------------------------------------//

import React, { useEffect } from 'react';
import { makeStyles } from '@material-ui/core/styles';
import Container from '@material-ui/core/Container';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import Typography from '@material-ui/core/Typography';
import Box from '@material-ui/core/Box';
import Card from '@material-ui/core/Card';
import CardHeader from '@material-ui/core/CardHeader';
import CardContent from '@material-ui/core/CardContent';
import { connect } from 'react-redux';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faUserCog } from '@fortawesome/free-solid-svg-icons';
import { library } from '@fortawesome/fontawesome-svg-core';

import CommonApi from '../../Api';

import Password from './components/Password';
import Users from './components/Users';

//---------------------------------------------------------------------------//

library.add(faUserCog);

const useStyles = makeStyles(theme => ({
    container:{
        marginTop: 15
    },
    card:{
        paddingTop: 10,
    },
    cardContent:{
        flexGrow: 1,
        backgroundColor: theme.palette.background.paper,
        display: 'flex',
        overflow: 'auto'
    },
    tabs: {
        borderRight: `1px solid ${theme.palette.divider}`,
        minWidth: 150,
        [theme.breakpoints.down('xs')]: {
            minWidth: 110,
        }
    },
}));

function TabPanel(props) {
    const { children, value, index, ...other } = props;

    return (
        <Typography
            component='div'
            role='tabpanel'
            hidden={value !== index}
            id={`vertical-tabpanel-${index}`}
            aria-labelledby={`vertical-tab-${index}`}
            {...other}
        >
            {value === index && <Box p={3}>{children}</Box>}
        </Typography>
    );
}

function a11yProps(index) {
    return {
        id: `vertical-tab-${index}`,
        'aria-controls': `vertical-tabpanel-${index}`,
    };
}

//---------------------------------------------------------------------------//
// Component
//---------------------------------------------------------------------------//

const Component = (props) => {

    const classes = useStyles();
    const [value, setValue] = React.useState(0);

    const handleChange = (event, newValue) => {
        setValue(newValue);
    };

    useEffect(() => {
        props.dispatchUtil.fetchUsers();
    },[props.dispatchUtil]); 

    return (
        <Container className={classes.container} maxWidth='md'>
            <Card className={classes.card}>
                <CardHeader 
                    avatar={
                        <FontAwesomeIcon icon='user-cog' size='lg' fixedWidth />
                    }
                    title='User Management'
                    style={{
                        borderBottom: '1px #dde0e6 solid',
                        paddingTop: 5,
                        paddingBottom: 10,
                        paddingLeft: 15,
                        color: '#495057'
                    }}
                >
                </CardHeader>
                <CardContent className={classes.cardContent}>
                    <Tabs
                        orientation='vertical'
                        variant='scrollable'
                        value={value}
                        onChange={handleChange}
                        aria-label='Vertical tabs example'
                        className={classes.tabs}
                    >
                        <Tab label='All Users' {...a11yProps(0)} />
                        <Tab label='Current User' {...a11yProps(1)} />
                    </Tabs>
                    <TabPanel value={value} index={0} style={{width:'100%'}}>
                        <Users {...props} />
                    </TabPanel>
                    <TabPanel value={value} index={1} style={{width:'100%'}}>
                        <Password {...props} />
                    </TabPanel>
                </CardContent>
            </Card>

        </Container>
    );
}

//---------------------------------------------------------------------------//
// Redux Actions
//---------------------------------------------------------------------------//

const mapStateToProps = state => ({
    state: {
        app: state.app,
        users: state.users
    }
});

const mapDispatchToProps = dispatch => ({

    dispatchUtil: {

        fetchUsers: () => {
            CommonApi.User.getUsers(dispatch);
        },
        
        updateUser: (user) => {
            CommonApi.User.updateUser(dispatch, user);
        }
    }
});

//---------------------------------------------------------------------------//
// Export
//---------------------------------------------------------------------------//

export default connect(
    mapStateToProps,
    mapDispatchToProps
)(Component);

//---------------------------------------------------------------------------//